#!/usr/bin/env python3
# -*-coding:utf8-*-
import math
from typing_extensions import (
    Literal,
)
class ArmLowSpdFeedback:
    '''
    msg_v2_feedback
    
    驱动器信息高速反馈 0x6

    节点 ID:
        0x1~0x06
    CAN ID:
        0X261~0x266

    Args:
        can_id: canid,表示当前电机序号
        vol: 当前驱动器电压
        foc_temp: 驱动器温度
        motor_temp: 电机温度
        foc_status: 驱动器状态码
        bus_current: 当前驱动器电流,单位0.001A,1.5KG关节电机无母线电流采样,默认发送0
    
    位描述:
    
        Byte 0:电压高八位, uint16, 当前驱动器电压单位: 0.1V
        Byte 1:电压低八位
        Byte 2:驱动器温度高八位, int16, 单位: 1℃
        Byte 3:驱动器温度低八位
        Byte 4:电机温度,int8,单位: 1℃
        Byte 5:驱动器状态,uint8
            bit[0] 电源电压是否过低(0--正常; 1--过低)
            bit[1] 电机是否过温(0--正常; 1--过温)
            bit[2] 驱动器是否过流(0--正常; 1--过流)
            bit[3] 驱动器是否过温(0--正常; 1--过温)
            bit[4] 碰撞保护状态(0--正常; 1--触发保护)-7.25修改,之前为传感器状态
            bit[5] 驱动器错误状态(0: 正常; 1--错误)
            bit[6] 驱动器使能状态(1--使能; 0--失能)
            bit[7] 堵转保护状态(0--正常; 1--触发保护)-2024-7-25修改,之前为回零状态
        Byte 6:母线电流高八位,uint16,当前驱动器电流单位: 0.001A,1.5KG关节电机无母线电流采样,默认发送0
        Byte 7:母线电流低八位
    '''
    '''
    msg_v2_feedback
    
    High-Speed Feedback of Drive Information 0x6

    Node ID:
        0x1~0x06

    CAN IDs:
        0x261~0x266

    Args:
        can_id: CAN ID, representing the current motor number.
        vol: Current driver voltage.
        foc_temp: Driver temperature.
        motor_temp: Motor temperature.
        foc_status: Driver status.
        bus_current: Current driver current.
    
    Bit Definitions:
    
        Byte 0: Bus Voltage (High Byte), uint16, unit: 0.1 V
        Byte 1: Bus Voltage (Low Byte)
        Byte 2: Drive Temperature (High Byte), int16, unit: 1°C
        Byte 3: Drive Temperature (Low Byte)
        Byte 4: Motor Temperature, int8, unit: 1°C
        Byte 5: Drive Status, uint8:
            bit[0]: Power voltage low (0: Normal, 1: Low)
            bit[1]: Motor over-temperature (0: Normal, 1: Over-temperature)
            bit[2]: Drive over-current (0: Normal, 1: Over-current)
            bit[3]: Drive over-temperature (0: Normal, 1: Over-temperature)
            bit[4]: Collision protection status (0: Normal, 1: Trigger protection) (Updated 7.25, previously sensor status)
            bit[5]: Drive error status (0: Normal, 1: Error)
            bit[6]: Drive enable status (1: Enabled, 0: Disabled)
            bit[7]: Stalling protection status (0: Normal, 1: Trigger protection) (Updated 7.25, previously zeroing status)
        Byte 6: Bus Current (High Byte), uint16, unit: 0.001 A, The 1.5KG joint motor has no bus current sampling and defaults to sending 0.
        Byte 7: Bus Current (Low Byte)
    '''
    def __init__(self, 
                 can_id: Literal[0x000, 0x261, 0x262, 0x263, 0x264, 0x264, 0x265, 0x266] = 0,
                 vol: int = 0, 
                 foc_temp: int = 0, 
                 motor_temp: int = 0,
                 foc_status: int = 0,
                 bus_current: int = 0,
                 ):
        if can_id not in [0x000, 0x261, 0x262, 0x263, 0x264, 0x264, 0x265, 0x266]:
            raise ValueError(f"'can_id' Value {can_id} out of range [0x000, 0x261, 0x262, 0x263, 0x264, 0x264, 0x265, 0x266]")
        self.can_id = can_id
        self.vol = vol
        self.foc_temp = foc_temp
        self.motor_temp = motor_temp
        self._foc_status_code = foc_status
        self.foc_status = self.FOC_Status()
        self.bus_current = bus_current

    class FOC_Status:
        def __init__(self):
            self.voltage_too_low  = False
            self.motor_overheating = False
            self.driver_overcurrent = False
            self.driver_overheating = False
            self.collision_status = False
            self.driver_error_status = False
            self.driver_enable_status = False
            self.stall_status  = False
        def __str__(self): 
            return (f"    voltage_too_low : {self.voltage_too_low}\n"
                    f"    motor_overheating: {self.motor_overheating}\n"
                    f"    driver_overcurrent: {self.driver_overcurrent}\n"
                    f"    driver_overheating: {self.driver_overheating}\n"
                    f"    collision_status: {self.collision_status}\n"
                    f"    driver_error_status: {self.driver_error_status}\n"
                    f"    driver_enable_status: {self.driver_enable_status}\n"
                    f"    stall_status: {self.stall_status}\n"
                    )

    @property
    def foc_status_code(self):
        return self._foc_status_code

    @foc_status_code.setter
    def foc_status_code(self, value: int):
        if not (0 <= value < 2**8):
            raise ValueError("foc_status_code must be an 8-bit integer between 0 and 255.")
        self._foc_status_code = value
        # Update foc_status based on the foc_status_code bits
        self.foc_status.voltage_too_low = bool(value & (1 << 0))
        self.foc_status.motor_overheating = bool(value & (1 << 1))
        self.foc_status.driver_overcurrent = bool(value & (1 << 2))
        self.foc_status.driver_overheating = bool(value & (1 << 3))
        self.foc_status.collision_status = bool(value & (1 << 4)) # 碰撞状态
        self.foc_status.driver_error_status = bool(value & (1 << 5))
        self.foc_status.driver_enable_status = bool(value & (1 << 6))
        self.foc_status.stall_status = bool(value & (1 << 7)) # 堵转状态

    def __str__(self):
        return (f"ArmLowSpdFeedback(\n"
                f"  can_id: {hex(self.can_id)},\n"
                f"  vol: {self.vol}, {self.vol*0.1:.1f}V,\n"
                f"  foc_temp: {self.foc_temp }C,\n"
                f"  motor_temp: {self.motor_temp }C,\n"
                f"  foc_status: \n{self.foc_status },\n"
                f"  bus_current: {self.bus_current}, {self.bus_current*0.001:.1f}A\n"
                f")")

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-
# can总线读取二次封装
import can
from can.message import Message
import time
from threading import Timer
import subprocess
from typing import (
    Callable,
    Iterator,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
    cast,
)

class C_STD_CAN():
    '''
    基础CAN数据帧的收发,内无线程创建,需要在类外调用的时候创建线程来循环read
    
    Args:
        channel_name: can的端口名称
        bustype: can总线类型,默认为socket can
        expected_bitrate: 预期can总线的波特率
        judge_flag: 是否在实例化该类时进行can端口判断,有些情况需要False 
        auto_init: 是否自动初始化can,也就是实例化can.interface.Bus
        callback_function: ReadCanMessage中的回调函数,应传入函数
    '''
    '''
    Basic CAN Frame Send/Receive with Thread Creation

    When calling outside the class, a thread needs to be created to continuously read the CAN data.

    Args:
        channel_name: The name of the CAN port.
        bustype: The type of CAN bus, default is socket CAN.
        expected_bitrate: The expected bitrate for the CAN bus.
        judge_flag: Whether to check the CAN port during the instantiation of the class. In some cases, it should be set to False.
        auto_init: Whether to automatically initialize the CAN bus (i.e., instantiate can.interface.Bus).
        callback_function: The callback function in ReadCanMessage, which should be passed as a function.
    '''
    def __init__(self, 
                 channel_name:str="can0", 
                 bustype="socketcan", 
                 expected_bitrate:int=1000000,
                 judge_flag:bool=True, 
                 auto_init:bool=True,
                 callback_function: Callable = None) -> None:
        self.channel_name = channel_name
        self.bustype = bustype
        self.expected_bitrate = expected_bitrate
        self.rx_message:Optional[Message]   #创建消息接收类
        self.callback_function = callback_function  #接收回调函数
        self.bus = None
        if(judge_flag):
            self.JudgeCanInfo()
        if(auto_init):
            self.Init()#创建can总线交互
        
    def __del__(self):
        try:
            self.bus.shutdown()  # 关闭 CAN 总线
            # print("CAN bus connection properly shut down.")
        except AttributeError:
            print("CAN bus connection was not properly initialized.")
        except Exception as e:
            print(f"Error occurred while shutting down CAN bus: {e}")
    
    def Init(self):
        '''初始化can总线
        '''
        '''Initialize the CAN bus.
        '''
        if self.bus is not None:
            return
        try:
            self.bus = can.interface.Bus(channel=self.channel_name, bustype=self.bustype)
            print(self.channel_name,"bus opened successfully.")
        except can.CanError as e:
            print(f"Failed to open CAN bus: {e}")
            self.bus = None

    def Close(self):
        '''关闭can总线
        '''
        '''Close the CAN bus.
        '''
        if self.bus is not None:
            try:
                self.bus.shutdown()  # 关闭 CAN 总线
                # print("CAN bus connection properly shut down.")
            except AttributeError:
                print("CAN bus connection was not properly initialized.")
                return -1
            except Exception as e:
                print(f"Error occurred while shutting down CAN bus: {e}")
                return -2
            self.bus = None
            # print("CAN bus closed successfully.")
            return 1
        else:
            print("CAN bus was not open.")
            return 0
    
    def JudgeCanInfo(self):
        '''
        类初始化时是否检测基础信息
        '''
        '''
        Whether to check basic information during class initialization.
        '''
        # 检查 CAN 端口是否存在
        if not self.is_can_socket_available(self.channel_name):
            raise ValueError(f"CAN socket {self.channel_name} does not exist.")
        print(self.channel_name, " is exist")
        # 检查 CAN 端口是否 UP
        if not self.is_can_port_up(self.channel_name):
            raise RuntimeError(f"CAN port {self.channel_name} is not UP.")
        print(self.channel_name, " is UP")
        # 检查 CAN 端口的比特率
        actual_bitrate = self.get_can_bitrate(self.channel_name)
        if self.expected_bitrate is not None and not (actual_bitrate == self.expected_bitrate):
            raise ValueError(f"CAN port {self.channel_name} bitrate is {actual_bitrate} bps, expected {self.expected_bitrate} bps.")
        print(self.channel_name, " bitrate is ", self.expected_bitrate)
    
    def GetBirtrate(self):
        return self.expected_bitrate

    def GetRxMessage(self):
        return self.rx_message

    def ReadCanMessage(self):
        if self.is_can_bus_ok():
            self.rx_message = self.bus.recv()
            if self.rx_message and self.callback_function:
                self.callback_function(self.rx_message) #回调函数处理接收的原始数据
        else:
            print("CAN bus is not OK, skipping message read")

    def SendCanMessage(self, arbitration_id, data):
        '''can transmit

        Args:
            arbitration_id (_type_): _description_
            data (_type_): _description_
            is_extended_id_ (bool, optional): _description_. Defaults to False.
        '''
        message = can.Message(channel=self.channel_name,
                              arbitration_id=arbitration_id, 
                              data=data, 
                              dlc=8,
                              is_extended_id=False)
        if self.is_can_bus_ok():
            try:
                self.bus.send(message)
                # print(message)
                # print(f"Message sent on {self.bus.channel_info}")
            except can.CanError:
                print(can.CanError,"Message NOT sent")
        else:
            print("CAN bus is not OK, cannot send message")

    def is_can_bus_ok(self) -> bool:
        '''
        检查CAN总线状态是否正常。
        '''
        '''
        Check whether the CAN bus status is normal.
        '''
        if isinstance(self.bus, can.BusABC):
            bus_state = self.bus.state
        else: bus_state = None
        if bus_state == can.BusState.ACTIVE:
            # print("CAN bus state: ACTIVE - Bus is functioning normally")
            return True
        elif bus_state == can.BusState.PASSIVE:
            print("CAN bus state: PASSIVE - Warning level errors are occurring")
            return False  # 可以根据需要调整
        elif bus_state == can.BusState.ERROR:
            print("CAN bus state: ERROR - Communication may be impaired")
            return False
        else:
            print(f"Unknown CAN bus state: {bus_state}")
            return False
    
    def is_can_socket_available(self, channel_name: str) -> bool:
        '''
        检查给定的 CAN 端口是否存在。
        '''
        '''
        Check if the given CAN port exists.
        '''
        try:
            with open(f"/sys/class/net/{channel_name}/operstate", "r") as file:
                state = file.read().strip()
                return state == "up"
        except FileNotFoundError:
            return False

    def get_can_ports(self) -> list:
        '''
        获取系统中所有可用的 CAN 端口。
        '''
        '''
        Get all available CAN ports in the system.
        '''
        import os
        can_ports = []
        for item in os.listdir('/sys/class/net/'):
            if 'can' in item:
                can_ports.append(item)
        return can_ports

    def can_port_info(self, channel_name: str) -> str:
        '''
        获取指定 CAN 端口的详细信息，包括状态、类型和比特率。
        '''
        '''
        Get detailed information about the specified CAN port, including status, type, and bit rate.
        '''
        try:
            with open(f"/sys/class/net/{channel_name}/operstate", "r") as file:
                state = file.read().strip()
            with open(f"/sys/class/net/{channel_name}/type", "r") as file:
                port_type = file.read().strip()
            bitrate = self.get_can_bitrate(channel_name)
            return f"CAN port {channel_name}: State={state}, Type={port_type}, Bitrate={bitrate} bps"
        except FileNotFoundError:
            return f"CAN port {channel_name} not found."

    def is_can_port_up(self, channel_name: str) -> bool:
        '''
        检查 CAN 端口是否为 UP 状态。
        '''
        '''
        Check if the CAN port is in the UP state.
        '''
        try:
            with open(f"/sys/class/net/{channel_name}/operstate", "r") as file:
                state = file.read().strip()
                return state == "up"
        except FileNotFoundError:
            return False

    def get_can_bitrate(self, channel_name: str) -> str:
        '''
        获取指定 CAN 端口的比特率。
        '''
        '''
        Get the bit rate of the specified CAN port.
        '''
        try:
            result = subprocess.run(['ip', '-details', 'link', 'show', channel_name],
                                    stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                    universal_newlines=True, check=True)  # Python 3.6
                                    # capture_output=True, text=True)
            output = result.stdout
            for line in output.split('\n'):
                if 'bitrate' in line:
                    return int(line.split('bitrate ')[1].split(' ')[0])
            return "Unknown"
        except Exception as e:
            print(f"Error while getting bitrate: {e}")
            return "Unknown"

# 示例代码
# if __name__ == "__main__":
#     try:
#         can_obj = C_STD_CAN(channel_name="can0")
#         print("CAN bus initialized successfully.")
#         print(can_obj.get_can_ports())
#         print(can_obj.can_port_info("can0"))
#     except ValueError as e:
#         print(e)
#     except Exception as e:
#         print(f"An unexpected error occurred: {e}")


#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# 读取机械臂的所有电机的最大加速度限制

from typing import (
    Optional,
)
import time
from piper_sdk import *

# 测试代码
if __name__ == "__main__":
    piper = C_PiperInterface()
    piper.ConnectPort()
    while True:
        # piper.SearchMotorMaxAngleSpdAccLimit(1,0x02)
        piper.SearchAllMotorMaxAngleSpd()
        # piper.SearchAllMotorMaxAccLimit()
        
        # print(piper.GetCurrentMotorMaxAccLimit())
        # print(piper.GetCurrentMotorAngleLimitMaxVel())
        print(piper.GetAllMotorAngleLimitMaxSpd())
        # print(piper.GetAllMotorMaxAccLimit())
        time.sleep(0.01)


#!/usr/bin/env python3
# -*-coding:utf8-*-
from typing_extensions import (
    Literal,
)

class ArmMsgCircularPatternCoordNumUpdateCtrl:
    '''
    msg_v2_transmit
    
    圆弧模式坐标序号更新指令
    
    CAN ID:
        0x158
    
    Args:
        instruction_num: 指令点序号
    
    位描述:

        Byte 0: uint8, 指令点序号,以整数表示。
                0x00 无效
                0x01 起点
                0x02 中点
                0x03 终点
    '''
    '''
    msg_v2_transmit
    
    Arc Mode Coordinate Index Update Command

    CAN ID:
        0x158

    Args:
        instruction_num: Instruction point index.

    Bit Description:

        Byte 0 instruction_num: uint8, Instruction point index, represented as an integer.
                        0x00 Invalid
                        0x01 Start point
                        0x02 Midpoint
                        0x03 Endpoint
    '''
    def __init__(self, instruction_num: Literal[0x00, 0x01, 0x02, 0x03] = 0x00):
        if instruction_num not in [0x00, 0x01, 0x02, 0x03]:
            raise ValueError(f"'instruction_num' Value {instruction_num} out of range [0x00, 0x01, 0x02, 0x03]")
        self.instruction_num = instruction_num

    def __str__(self):
        return (f"ArmMsgCircularPatternCoordNumUpdateCtrl(\n"
                f"  instruction_num: {self.instruction_num},\n"
                f")")

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-

from typing import (
    Optional,
)
import time
from piper_sdk import *

# 测试代码
if __name__ == "__main__":
    piper = C_PiperInterface_V2(dh_is_offset=1)
    piper.ConnectPort()
    while True:
        print(f"feedback:{piper.GetFK('feedback')[-1]}")
        # print(f"control:{piper.GetFK('control')}")
        time.sleep(0.01)


#!/usr/bin/env python3
# -*-coding:utf8-*-

class ArmMsgJointMitCtrl():
    '''
    msg_v2_transmit
    
    机械臂关节mit控制
    
    CAN ID:
        0x15A,0x15B,0x15C,0x15D,0x15E,0x15F
    
    每个ID对应单个关节,因此有六个ID
    
    Args:
        pos_ref: 设定期望的目标位置
        vel_ref: 设定电机运动的速度
        kp: 比例增益，控制位置误差对输出力矩的影响
        kd: 微分增益，控制速度误差对输出力矩的影响
        t_ref: 目标力矩参考值，用于控制电机施加的力矩或扭矩
        crc: 循环冗余校验，用于数据完整性验证
    
    位描述:
    
        Byte 0: Pos_ref [bit15~bit8] 高8位
        Byte 1: Pos_ref [bit7~bit0]  低8位
        Byte 2: Vel_ref [bit11~bit4] 低12位
        Byte 3: Vel_ref [bit3~bit0], Kp [bit11~bit8]
        Byte 4: Kp [bit7~bit0],      Kp给定参考值: 10
        Byte 5: Kd [bit11~bit4]      低12位,Kd给定参考值: 0.8
        Byte 6: Kd [bit3~bit0] T_ref [bit7~bit4]
        Byte 7: T_ref [bit3~bit0] CRC [bit3~bit0]
    '''
    '''
    msg_v2_transmit
    
    Mechanical Arm Joint MIT Control

    CAN IDs:
        0x15A, 0x15B, 0x15C, 0x15D, 0x15E, 0x15F

    Each ID corresponds to a single joint, thus there are six IDs.
    
    Args:
        pos_ref: Desired target position
        vel_ref: Desired motor motion speed
        kp: Proportional gain, controls the influence of position error on output torque
        kd: Derivative gain, controls the influence of velocity error on output torque
        t_ref: Target torque reference value, used to control the motor's applied force or torque
        crc: Cyclic Redundancy Check for data integrity verification
    
    Bit Description:
    
        Byte 0	Pos_ref	bit15~bit8	High 8 bits of pos_ref
        Byte 1	Pos_ref	bit7~bit0	Low 8 bits of pos_ref
        Byte 2	Vel_ref	bit11~bit4	Low 12 bits of vel_ref
        Byte 3	Vel_ref, Kp	bit3~bit0, bit11~bit8	Remaining 4 bits of vel_ref, high 4 bits of kp
        Byte 4	Kp	bit7~bit0	Low 8 bits of kp (default: 10)
        Byte 5	Kd	bit11~bit4	Low 12 bits of kd (default: 0.8)
        Byte 6	Kd, T_ref	bit3~bit0, bit7~bit4	Remaining 4 bits of kd, high 4 bits of t_ref
        Byte 7	T_ref, CRC	bit3~bit0, bit3~bit0	Low 4 bits of t_ref, low 4 bits of crc
    '''
    def __init__(self, 
                 pos_ref = 0, 
                 vel_ref = 0, 
                 kp = 10, 
                 kd = 0.8,
                 t_ref = 0, 
                 crc = 0):
        self.pos_ref = pos_ref
        self.vel_ref = vel_ref
        self.kp = kp
        self.kd = kd
        self.t_ref = t_ref
        self.crc = crc
    
    def __str__(self):
        # 将角度乘以0.001，并保留三位小数
        mit_args = [
            ("pos_ref", self.pos_ref),
            ("vel_ref", self.vel_ref ),
            ("kp", self.kp ),
            ("kd", self.kd ),
            ("t_ref", self.t_ref ),
            ("crc", self.crc )
        ]

        # 生成格式化字符串，保留三位小数
        formatted_str = "\n".join([f"{name}: {param}" for name, param in mit_args])
        
        return f"ArmMsgJointMitCtrl:\n{formatted_str}"
    
    def __repr__(self):
        return self.__str__()

class ArmMsgAllJointMitCtrl:
    '''
    msg_v2_transmit
    
    机械臂关节mit控制,全部关节
    
    CAN ID:
        0x15A,0x15B,0x15C,0x15D,0x15E,0x15F
    
    每个ID对应单个关节,因此有六个ID
    
    Args:
        pos_ref: 设定期望的目标位置
        vel_ref: 设定电机运动的速度
        kp: 比例增益，控制位置误差对输出力矩的影响
        kd: 微分增益，控制速度误差对输出力矩的影响
        t_ref: 目标力矩参考值，用于控制电机施加的力矩或扭矩
        crc: 循环冗余校验，用于数据完整性验证
    
    位描述:
    
        Byte 0: Pos_ref [bit15~bit8] 高8位
        Byte 1: Pos_ref [bit7~bit0]  低8位
        Byte 2: Vel_ref [bit11~bit4] 低12位
        Byte 3: Vel_ref [bit3~bit0], Kp [bit11~bit8]
        Byte 4: Kp [bit7~bit0],      Kp给定参考值: 10
        Byte 5: Kd [bit11~bit4]      低12位,Kd给定参考值: 0.8
        Byte 6: Kd [bit3~bit0] T_ref [bit7~bit4]
        Byte 7: T_ref [bit3~bit0] CRC [bit3~bit0]
    '''
    '''
    msg_v2_transmit
    
    Mechanical Arm Joint MIT Control

    CAN IDs:
        0x15A, 0x15B, 0x15C, 0x15D, 0x15E, 0x15F

    Each ID corresponds to a single joint, thus there are six IDs.
    
    Args:
        pos_ref: Desired target position
        vel_ref: Desired motor motion speed
        kp: Proportional gain, controls the influence of position error on output torque
        kd: Derivative gain, controls the influence of velocity error on output torque
        t_ref: Target torque reference value, used to control the motor's applied force or torque
        crc: Cyclic Redundancy Check for data integrity verification
    
    Bit Description:
    
        Byte 0	Pos_ref	bit15~bit8	High 8 bits of pos_ref
        Byte 1	Pos_ref	bit7~bit0	Low 8 bits of pos_ref
        Byte 2	Vel_ref	bit11~bit4	Low 12 bits of vel_ref
        Byte 3	Vel_ref, Kp	bit3~bit0, bit11~bit8	Remaining 4 bits of vel_ref, high 4 bits of kp
        Byte 4	Kp	bit7~bit0	Low 8 bits of kp (default: 10)
        Byte 5	Kd	bit11~bit4	Low 12 bits of kd (default: 0.8)
        Byte 6	Kd, T_ref	bit3~bit0, bit7~bit4	Remaining 4 bits of kd, high 4 bits of t_ref
        Byte 7	T_ref, CRC	bit3~bit0, bit3~bit0	Low 4 bits of t_ref, low 4 bits of crc
    '''
    def __init__(self, 
                 m1:ArmMsgJointMitCtrl=ArmMsgJointMitCtrl(0,0,10,0.8,0,0), 
                 m2:ArmMsgJointMitCtrl=ArmMsgJointMitCtrl(0,0,10,0.8,0,0),
                 m3:ArmMsgJointMitCtrl=ArmMsgJointMitCtrl(0,0,10,0.8,0,0), 
                 m4:ArmMsgJointMitCtrl=ArmMsgJointMitCtrl(0,0,10,0.8,0,0), 
                 m5:ArmMsgJointMitCtrl=ArmMsgJointMitCtrl(0,0,10,0.8,0,0), 
                 m6:ArmMsgJointMitCtrl=ArmMsgJointMitCtrl(0,0,10,0.8,0,0)
                 ):
        self.__m = [ArmMsgJointMitCtrl(0,0,10,0.8,0,0), m1, m2, m3, m4, m5, m6]
        self.motor = [ArmMsgJointMitCtrl(0,0,10,0.8,0,0) for _ in range(7)]

    def assign(self):
        for i in range(1,7):
            if(self.__m[i].joint_motor_num != 0):
                self.motor[i] = self.__m[i]
    
    def __str__(self):
        return (f"{self.motor[1]}\n"
                f"{self.motor[2]}\n"
                f"{self.motor[3]}\n"
                f"{self.motor[4]}\n"
                f"{self.motor[5]}\n"
                f"{self.motor[6]}\n"
                f")")

    def __repr__(self):
        return self.__str__()


#!/usr/bin/env python3
# -*-coding:utf8-*-
import math
from typing_extensions import (
    Literal,
)
class ArmMsgFeedbackJointVelAcc:
    '''
    msg_v1_feedback
    
    反馈各个关节当前末端速度/加速度

    CAN ID:
        0x481 ~ 0x486
        代表 1~6 号关节

    Args:
        can_id: 当前canid,用来代表关节序号
        end_linear_vel: 末端线速度
        end_angular_vel: 末端角速度
        end_linear_acc: 末端线加速度
        end_angular_acc: 末端角加速度
    
    位描述:

        Byte 0: 末端线速度 H, uint16, 单位 0.001m/s
        Byte 1: 末端线速度 L,
        Byte 2: 末端角速度 H, uint16, 单位 0.001rad/s
        Byte 3: 末端角速度 L,
        Byte 4: 末端线加速度 H, uint16, 单位 0.001m/s^2
        Byte 5: 末端线加速度 L
        Byte 6: 末端角加速度 H, uint16, 单位 0.001rad/s^2
        Byte 7: 末端角加速度 L
    '''
    '''
    msg_v1_feedback
    
    Feedback on Current End-Effector Speed/Acceleration for Each Joint

    CAN ID:
        0x481 ~ 0x486
        Corresponds to Joints 1~6.

    Args:
        can_id: Current CAN ID, used to represent the joint number.
        end_linear_vel: End-effector linear velocity.
        end_angular_vel: End-effector angular velocity.
        end_linear_acc: End-effector linear acceleration.
        end_angular_acc: End-effector angular acceleration.
    
    Bit Description:

        Byte 0: End-Effector Linear Velocity (High Byte), uint16, unit: 0.001 m/s
        Byte 1: End-Effector Linear Velocity (Low Byte)
        Byte 2: End-Effector Angular Velocity (High Byte), uint16, unit: 0.001 rad/s
        Byte 3: End-Effector Angular Velocity (Low Byte)
        Byte 4: End-Effector Linear Acceleration (High Byte), uint16, unit: 0.001 m/s²
        Byte 5: End-Effector Linear Acceleration (Low Byte)
        Byte 6: End-Effector Angular Acceleration (High Byte), uint16, unit: 0.001 rad/s²
        Byte 7: End-Effector Angular Acceleration (Low Byte)
    '''
    def __init__(self, 
                 can_id: Literal[0, 0x481, 0x482, 0x483, 0x484, 0x485, 0x486] = 0,
                 end_linear_vel: int = 0, 
                 end_angular_vel: int = 0, 
                 end_linear_acc: int = 0,
                 end_angular_acc: int = 0
                 ):
        if can_id not in [0, 0x481, 0x482, 0x483, 0x484, 0x485, 0x486]:
            raise ValueError(f"'can_id' Value {can_id} out of range [0x481, 0x482, 0x483, 0x484, 0x485, 0x486]")
        self.can_id = can_id
        self.end_linear_vel = end_linear_vel
        self.end_angular_vel = end_angular_vel
        self.end_linear_acc = end_linear_acc
        self.end_angular_acc = end_angular_acc

    def __str__(self):
        return (f"ArmMsgFeedbackJointVelAcc(\n"
                f"  can_id: {self.can_id},\n"
                f"  end_linear_vel: {self.end_linear_vel},\n"
                f"  end_angular_vel: {self.end_angular_vel },\n"
                f"  end_linear_acc: {self.end_linear_acc },\n"
                f"  end_angular_acc: {self.end_angular_acc}\n"
                f")")

    def __repr__(self):
        return self.__str__()

class ArmMsgFeedbackAllJointVelAcc:
    '''
    反馈全部关节当前末端速度/加速度

    CAN ID:
        0x481 ~ 0x486
        代表 1~6 号关节

    Args:
        j1: 电机1的当前末端速度/加速度
        j2: 电机2的当前末端速度/加速度
        j3: 电机3的当前末端速度/加速度
        j4: 电机4的当前末端速度/加速度
        j5: 电机5的当前末端速度/加速度
        j6: 电机6的当前末端速度/加速度
    '''
    '''
    Feedback on Current End-Effector Speed/Acceleration for Each Joint

    CAN ID:
        0x481 ~ 0x486
        Corresponds to Joints 1~6.

    Args:
        j1: Current end-effector velocity/acceleration for motor 1.
        j2: Current end-effector velocity/acceleration for motor 2.
        j3: Current end-effector velocity/acceleration for motor 3.
        j4: Current end-effector velocity/acceleration for motor 4.
        j5: Current end-effector velocity/acceleration for motor 5.
        j6: Current end-effector velocity/acceleration for motor 6.
    '''
    def __init__(self, 
                 j1:ArmMsgFeedbackJointVelAcc = ArmMsgFeedbackJointVelAcc(0,0,0,0,0),
                 j2:ArmMsgFeedbackJointVelAcc = ArmMsgFeedbackJointVelAcc(0,0,0,0,0),
                 j3:ArmMsgFeedbackJointVelAcc = ArmMsgFeedbackJointVelAcc(0,0,0,0,0),
                 j4:ArmMsgFeedbackJointVelAcc = ArmMsgFeedbackJointVelAcc(0,0,0,0,0),
                 j5:ArmMsgFeedbackJointVelAcc = ArmMsgFeedbackJointVelAcc(0,0,0,0,0),
                 j6:ArmMsgFeedbackJointVelAcc = ArmMsgFeedbackJointVelAcc(0,0,0,0,0)):
        self.j = [ArmMsgFeedbackJointVelAcc(0,0,0,0,0), j1, j2, j3, j4, j5, j6]
        self.joint = [ArmMsgFeedbackJointVelAcc() for _ in range(7)]
        self.joint[0] = ArmMsgFeedbackJointVelAcc(0,0,0,0,0)
    
    def assign(self):
        for i in range(1,7):
            if(self.j[i].can_id != 0):
                self.joint[i] = self.j[i]
    
    def __str__(self):
        return (f"{self.joint[1]}\n"
                f"{self.joint[2]}\n"
                f"{self.joint[3]}\n"
                f"{self.joint[4]}\n"
                f"{self.joint[5]}\n"
                f"{self.joint[6]}\n"
                f")")

    def __repr__(self):
        return self.__str__()


#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# 设置机械臂为从动臂
# 注意，如果是在机械臂处于主动臂模式下，发送设置指令后需要重新启动机械臂
from typing import (
    Optional,
)
import time
from piper_sdk import *

# 测试代码
if __name__ == "__main__":
    piper = C_PiperInterface_V2()
    piper.ConnectPort()
    piper.MasterSlaveConfig(0xFC, 0, 0, 0)



#!/usr/bin/env python3
# -*-coding:utf8-*-
from typing_extensions import (
    Literal,
)
class ArmMsgMotionCtrl_2():
    '''
    msg_v1_transmit
    
    机械臂运动控制指令2

    CAN ID:
        0x151
    
    Args:
        ctrl_mode: 控制模式
        move_mode: MOVE模式
        move_spd_rate_ctrl: 运动速度百分比
        mit_mode: mit模式
        residence_time: 离线轨迹点停留时间
    
    位描述:
    
        Byte 0: 控制模式     uint8
                0x00 待机模式
                0x01 CAN 指令控制模式
                0x03 以太网控制模式
                0x04 wifi 控制模式
                0x07 离线轨迹模式
        Byte 1: MOVE模式     uint8    
                0x00 MOVE P
                0x01 MOVE J
                0x02 MOVE L
                0x03 MOVE C
        Byte 2: 运动速度百分比 uint8    0~100
        Byte 3: mit模式      uint8   
                0x00 位置速度模式
                0xAD MIT模式
                0xFF 无效
        Byte 4: 离线轨迹点停留时间 uint8 0~254 ,单位: s;255:轨迹终止
    '''
    '''
    msg_v1_transmit
    
    Robotic Arm Motion Control Command 2

    CAN ID:
        0x151
    
    Args:
        ctrl_mode: Control mode.
        move_mode: MOVE mode.
        move_spd_rate_ctrl: Movement speed as a percentage.
        mit_mode: MIT mode.
        residence_time: Hold time at offline trajectory points.

    Bit Descriptions:

        Byte 0 control_mode: uint8, control mode selection.
            0x00: Standby mode.
            0x01: CAN command control mode.
            0x03: Ethernet control mode.
            0x04: Wi-Fi control mode.
            0x07: Offline trajectory mode.

        Byte 1 move_mode: uint8, movement mode selection.
            0x00: MOVE P (Position).
            0x01: MOVE J (Joint).
            0x02: MOVE L (Linear).
            0x03: MOVE C (Circular).

        Byte 2 speed_percentage: uint8, movement speed as a percentage.
            Range: 0~100.

        Byte 3 mit_mode: uint8, motion control mode.
            0x00: Position-speed mode.
            0xAD: MIT mode.
            0xFF: Invalid.

        Byte 4 offline_trajectory_hold_time: uint8, duration to hold at offline trajectory points.
            Range: 0~255, unit: seconds.
    '''
    def __init__(self, 
                 ctrl_mode: Literal[0x00, 0x01, 0x03, 0x04, 0x07] = 0x01, 
                 move_mode: Literal[0x00, 0x01, 0x02, 0x03] = 0x01, 
                 move_spd_rate_ctrl: int = 50,
                 mit_mode: Literal[0x00, 0xAD, 0xFF] = 0x00,
                 residence_time: int = 0):
        # 检查是否在有效范围内
        if ctrl_mode not in [0x00, 0x01, 0x03, 0x04, 0x07]:
            raise ValueError(f"'ctrl_mode' Value {ctrl_mode} out of range [0x00, 0x01, 0x02, 0x03, 0x04, 0x07]")
        if move_mode not in [0x00, 0x01, 0x02, 0x03]:
            raise ValueError(f"'move_mode' Value {move_mode} out of range [0x00, 0x01, 0x02, 0x03]")
        if not (0 <= move_spd_rate_ctrl <= 100):
            raise ValueError(f"'move_spd_rate_ctrl' Value {move_spd_rate_ctrl} out of range 0-100")
        if mit_mode not in [0x00, 0xAD, 0xFF]:
            raise ValueError(f"'mit_mode' Value {mit_mode} out of range [0x00, 0xAD, 0xFF]")
        if not (0 <= residence_time <= 255):
            raise ValueError(f"'residence_time' Value {residence_time} out of range 0-255")
        self.ctrl_mode = ctrl_mode
        self.move_mode = move_mode
        self.move_spd_rate_ctrl = move_spd_rate_ctrl
        self.mit_mode = mit_mode
        self.residence_time = residence_time

    def __str__(self):
        dict_ = [
            (" ctrl_mode ", self.ctrl_mode),
            (" move_mode ", self.move_mode),
            (" move_spd_rate_ctrl ", self.move_spd_rate_ctrl),
            (" mit_mode ", self.mit_mode),
            (" residence_time ", self.residence_time)
        ]

        # 生成格式化字符串，保留三位小数
        formatted_ = "\n".join([f"{name}: {value}" for name, value in dict_])
        
        return f"ArmMsgMotionCtrl_2:\n{formatted_}"
    
    def __repr__(self):
        return self.__str__()


#!/usr/bin/env python3
# -*-coding:utf8-*-
from typing_extensions import (
    Literal,
)
class ArmMsgFeedbackCurrentMotorMaxAccLimit:
    '''
    msg_v1_feedback
    
    反馈当前电机最大加速度限制

    CAN ID:
        0x47C

    Args:
        joint_motor_num: 关节电机序号
        max_joint_acc: 最大关节加速度
    
    位描述:

        Byte 0: 关节序号, uint8, 值域 1-6(1-6 代表关节驱动器序号)
        Byte 1: 最大关节加速度 H, uint16, 单位 0.001rad/^2
        Byte 2: 最大关节加速度 L
    '''
    '''
    msg_v1_feedback
    
    Feedback on Current Motor Maximum Acceleration Limit

    CAN ID: 
        0x47C

    Args:
        joint_motor_num: Joint motor number.
        max_joint_acc: Maximum joint acceleration.
    
    Bit Description:

        Byte 0: Joint Index, uint8, range 1-6(1-6 represent the joint motor index)
        Byte 1: Maximum Joint Acceleration (High Byte), uint16, unit: 0.001rad/^2
        Byte 2: Maximum Joint Acceleration (Low Byte)
    '''
    def __init__(self, 
                 joint_motor_num: Literal[0, 1, 2, 3, 4, 5, 6] = 0, 
                 max_joint_acc: int = 0
                 ):
        if joint_motor_num not in [0, 1, 2, 3, 4, 5, 6]:
            raise ValueError(f"'joint_motor_num' Value {joint_motor_num} out of range [1, 2, 3, 4, 5, 6]")
        self.joint_motor_num = joint_motor_num
        self.max_joint_acc = max_joint_acc

    def __str__(self):
        return (f"ArmMsgFeedbackCurrentMotorMaxAccLimit(\n"
                f"  joint_motor_num: {self.joint_motor_num},\n"
                f"  max_joint_acc: {self.max_joint_acc}, {self.max_joint_acc*0.001} rad/^2\n"
                f")")

    def __repr__(self):
        return self.__str__()

class ArmMsgFeedbackAllCurrentMotorMaxAccLimit:
    '''
    反馈全部电机最大加速度限制

    CAN ID:
        0x47C

    :param m1: 电机1的最大加速度限制
    :param m2: 电机2的最大加速度限制
    :param m3: 电机3的最大加速度限制
    :param m4: 电机4的最大加速度限制
    :param m5: 电机5的最大加速度限制
    :param m6: 电机6的最大加速度限制
    '''
    '''
    Feedback on Current Motor Maximum Acceleration Limit

    CAN ID: 
        0x47C

    :param m1: Maximum acceleration limit for motor 1.
    :param m2: Maximum acceleration limit for motor 2.
    :param m3: Maximum acceleration limit for motor 3.
    :param m4: Maximum acceleration limit for motor 4.
    :param m5: Maximum acceleration limit for motor 5.
    :param m6: Maximum acceleration limit for motor 6.
    '''
    def __init__(self, 
                 m1:ArmMsgFeedbackCurrentMotorMaxAccLimit=ArmMsgFeedbackCurrentMotorMaxAccLimit(0,0), 
                 m2:ArmMsgFeedbackCurrentMotorMaxAccLimit=ArmMsgFeedbackCurrentMotorMaxAccLimit(0,0),
                 m3:ArmMsgFeedbackCurrentMotorMaxAccLimit=ArmMsgFeedbackCurrentMotorMaxAccLimit(0,0), 
                 m4:ArmMsgFeedbackCurrentMotorMaxAccLimit=ArmMsgFeedbackCurrentMotorMaxAccLimit(0,0), 
                 m5:ArmMsgFeedbackCurrentMotorMaxAccLimit=ArmMsgFeedbackCurrentMotorMaxAccLimit(0,0), 
                 m6:ArmMsgFeedbackCurrentMotorMaxAccLimit=ArmMsgFeedbackCurrentMotorMaxAccLimit(0,0)
                 ):
        self.__m = [ArmMsgFeedbackCurrentMotorMaxAccLimit(0,0), m1, m2, m3, m4, m5, m6]
        self.motor = [ArmMsgFeedbackCurrentMotorMaxAccLimit(0,0) for _ in range(7)]

    def assign(self):
        for i in range(1,7):
            if(self.__m[i].joint_motor_num != 0):
                self.motor[i] = self.__m[i]
    
    def __str__(self):
        return (f"{self.motor[1]}\n"
                f"{self.motor[2]}\n"
                f"{self.motor[3]}\n"
                f"{self.motor[4]}\n"
                f"{self.motor[5]}\n"
                f"{self.motor[6]}\n"
                f")")

    def __repr__(self):
        return self.__str__()


#!/usr/bin/env python3
# -*-coding:utf8-*-

class ArmMsgStatus:
    '''
    msg_v1_feedback
    
    机械臂状态

    CAN ID:
        0x2A1

    Args:
        ctrl_mode: 控制模式
        arm_status: 机械臂状态
        mode_feed: 模式反馈
        teach_status: 示教状态
        motion_status: 运动状态
        trajectory_num: 当前运行轨迹点序号
        err_code: 故障码
    
    位描述:

        Byte 0:控制模式,uint8 
            - 0x00 待机模式
            - 0x01 CAN指令控制模式
            - 0x02 示教模式
            - 0x03 以太网控制模式
            - 0x04 wifi控制模式
            - 0x05 遥控器控制模式
            - 0x06 联动示教输入模式
            - 0x07 离线轨迹模式
        Byte 1:机械臂状态,uint8 
            - 0x00 正常
            - 0x01 急停
            - 0x02 无解
            - 0x03 奇异点
            - 0x04 目标角度超过限
            - 0x05 关节通信异常
            - 0x06 关节抱闸未打开
            - 0x07 机械臂发生碰撞
            - 0x08 拖动示教时超速
            - 0x09 关节状态异常
            - 0x0A 其它异常
            - 0x0B 示教记录
            - 0x0C 示教执行
            - 0x0D 示教暂停
            - 0x0E 主控NTC过温
            - 0x0F 释放电阻NTC过温
        Byte 2:模式反馈,uint8 
            - 0x00 MOVE P
            - 0x01 MOVE J
            - 0x02 MOVE L
            - 0x03 MOVE C
            - 0x04 MOVE M
        Byte 3:示教状态,uint8 
            - 0x00 关闭
            - 0x01 开始示教记录（进入拖动示教模式）
            - 0x02 结束示教记录（退出拖动示教模式）
            - 0x03 执行示教轨迹（拖动示教轨迹复现）
            - 0x04 暂停执行
            - 0x05 继续执行（轨迹复现继续）
            - 0x06 终止执行
            - 0x07 运动到轨迹起点
        Byte 4:运动状态,uint8 
            - 0x00 到达指定点位
            - 0x01 未到达指定点位
        Byte 5:当前运行轨迹点序号,uint8_t
            - 0~255 (离线轨迹模式下反馈)
        Byte 6:故障码*,uint16
            - bit[0]      1号关节通信异常(0:正常 1:异常)
            - bit[1]      2号关节通信异常(0:正常 1:异常)
            - bit[2]      3号关节通信异常(0:正常 1:异常)
            - bit[3]      4号关节通信异常(0:正常 1:异常)
            - bit[4]      5号关节通信异常(0:正常 1:异常)
            - bit[5]      6号关节通信异常(0:正常 1:异常)
            - bit[6]      保留(Reserved)
            - bit[7]      保留(Reserved)
        Byte 7:故障码*,uint16
            - bit[0]      1号关节角度超限位(0:正常 1:异常)
            - bit[1]      2号关节角度超限位(0:正常 1:异常)
            - bit[2]      3号关节角度超限位(0:正常 1:异常)
            - bit[3]      4号关节角度超限位(0:正常 1:异常)
            - bit[4]      5号关节角度超限位(0:正常 1:异常)
            - bit[5]      6号关节角度超限位(0:正常 1:异常)
            - bit[6]      保留
            - bit[7]      保留
    '''
    '''
    msg_v1_feedback
    
    Robot Arm Status

    CAN ID: 
        0x2A1

    Arguments:
        ctrl_mode: Control mode
        arm_status: Robot arm status
        mode_feed: Mode feedback
        teach_status: Teaching status
        motion_status: Motion status
        trajectory_num: Current trajectory point number
        err_code: Error code
    
    Bit Description:

        Byte 0: Control mode, uint8
            - 0x00: Standby mode
            - 0x01: CAN instruction control mode
            - 0x02: Teaching mode
            - 0x03: Ethernet control mode
            - 0x04: Wi-Fi control mode
            - 0x05: Remote control mode
            - 0x06: Linkage teaching input mode
            - 0x07: Offline trajectory mode
        Byte 1: Robot arm status, uint8
            - 0x00: Normal
            - 0x01: Emergency stop
            - 0x02: No solution
            - 0x03: Singularity point
            - 0x04: Target angle exceeds limit
            - 0x05: Joint communication exception
            - 0x06: Joint brake not released
            - 0x07: Collision occurred
            - 0x08: Overspeed during teaching drag
            - 0x09: Joint status abnormal
            - 0x0A: Other exception
            - 0x0B: Teaching record
            - 0x0C: Teaching execution
            - 0x0D: Teaching pause
            - 0x0E: Main controller NTC over temperature
            - 0x0F: Release resistor NTC over temperature
        Byte 2: Mode feedback, uint8
            - 0x00: MOVE P
            - 0x01: MOVE J
            - 0x02: MOVE L
            - 0x03: MOVE C
            - 0x04: MOVE M
        Byte 3: Teaching status, uint8
            - 0x00: Off
            - 0x01: Start teaching record (enter drag teaching mode)
            - 0x02: End teaching record (exit drag teaching mode)
            - 0x03: Execute teaching trajectory (reproduce drag teaching trajectory)
            - 0x04: Pause execution
            - 0x05: Continue execution (continue trajectory reproduction)
            - 0x06: Terminate execution
            - 0x07: Move to trajectory starting point
        Byte 4: Motion status, uint8
            - 0x00: Reached the target position
            - 0x01: Not yet reached the target position
        Byte 5: Current trajectory point number, uint8_t
            - 0~255 (feedback in offline trajectory mode)
        Byte 6: Error code, uint16
            - bit[0]: Joint 1 communication exception (0: normal, 1: abnormal)
            - bit[1]: Joint 2 communication exception (0: normal, 1: abnormal)
            - bit[2]: Joint 3 communication exception (0: normal, 1: abnormal)
            - bit[3]: Joint 4 communication exception (0: normal, 1: abnormal)
            - bit[4]: Joint 5 communication exception (0: normal, 1: abnormal)
            - bit[5]: Joint 6 communication exception (0: normal, 1: abnormal)
            - bit[6]: Reserved
            - bit[7]: Reserved
        Byte 7: Error code, uint16
            - bit[0]: Joint 1 angle limit exceeded (0: normal, 1: abnormal)
            - bit[1]: Joint 2 angle limit exceeded (0: normal, 1: abnormal)
            - bit[2]: Joint 3 angle limit exceeded (0: normal, 1: abnormal)
            - bit[3]: Joint 4 angle limit exceeded (0: normal, 1: abnormal)
            - bit[4]: Joint 5 angle limit exceeded (0: normal, 1: abnormal)
            - bit[5]: Joint 6 angle limit exceeded (0: normal, 1: abnormal)
            - bit[6]: Reserved
            - bit[7]: Reserved
    '''
    
    def __init__(self,
                 ctrl_mode: int = 0,
                 arm_status: int = 0,
                 mode_feed: int = 0,
                 teach_status: int = 0,
                 motion_status: int = 0,
                 trajectory_num: int = 0,
                 err_code: int = 0):
        self.ctrl_mode: int = ctrl_mode       #控制模式
        self.arm_status: int = arm_status      #机械臂状态
        self.mode_feed: int = mode_feed       #模式反馈
        self.teach_status: int = teach_status    #示教状态
        self.motion_status: int = motion_status   #运动状态
        self.trajectory_num: int = trajectory_num  #当前运行轨迹点序号
        self._err_code = err_code         #故障码
        self.err_status = self.ErrStatus()#故障码

    class ErrStatus:
        def __init__(self):
            self.joint_1_angle_limit = False
            self.joint_2_angle_limit = False
            self.joint_3_angle_limit = False
            self.joint_4_angle_limit = False
            self.joint_5_angle_limit = False
            self.joint_6_angle_limit = False
            self.communication_status_joint_1 = False
            self.communication_status_joint_2 = False
            self.communication_status_joint_3 = False
            self.communication_status_joint_4 = False
            self.communication_status_joint_5 = False
            self.communication_status_joint_6 = False
            
        def __str__(self):
            return (f" Joint 1 Angle Limit Status: {self.joint_1_angle_limit}\n"
                    f" Joint 2 Angle Limit Status: {self.joint_2_angle_limit}\n"
                    f" Joint 3 Angle Limit Status: {self.joint_3_angle_limit}\n"
                    f" Joint 4 Angle Limit Status: {self.joint_4_angle_limit}\n"
                    f" Joint 5 Angle Limit Status: {self.joint_5_angle_limit}\n"
                    f" Joint 6 Angle Limit Status: {self.joint_6_angle_limit}\n"
                    f" Joint 1 Communication Status: {self.communication_status_joint_1}\n"
                    f" Joint 2 Communication Status: {self.communication_status_joint_2}\n"
                    f" Joint 3 Communication Status: {self.communication_status_joint_3}\n"
                    f" Joint 4 Communication Status: {self.communication_status_joint_4}\n"
                    f" Joint 5 Communication Status: {self.communication_status_joint_5}\n"
                    f" Joint 6 Communication Status: {self.communication_status_joint_6}\n")

    @property
    def err_code(self):
        return self._err_code

    @err_code.setter
    def err_code(self, value: int):
        if not (0 <= value < 2**16):
            raise ValueError("err_code must be an 8-bit integer between 0 and 255.")
        self._err_code = value
        # Update err_status based on the err_code bits
        self.err_status.communication_status_joint_1 = bool(value & (1 << 0))
        self.err_status.communication_status_joint_2 = bool(value & (1 << 1))
        self.err_status.communication_status_joint_3 = bool(value & (1 << 2))
        self.err_status.communication_status_joint_4 = bool(value & (1 << 3))
        self.err_status.communication_status_joint_5 = bool(value & (1 << 4))
        self.err_status.communication_status_joint_6 = bool(value & (1 << 5))
        self.err_status.communication_status_joint_1 = bool(value & (1 << 8))
        self.err_status.communication_status_joint_2 = bool(value & (1 << 9))
        self.err_status.communication_status_joint_3 = bool(value & (1 << 10))
        self.err_status.communication_status_joint_4 = bool(value & (1 << 11))
        self.err_status.communication_status_joint_5 = bool(value & (1 << 12))
        self.err_status.communication_status_joint_6 = bool(value & (1 << 13))

    def __str__(self):
        return (f"Control Mode: {self.ctrl_mode}\n"
                f"Arm Status: {self.arm_status}\n"
                f"Mode Feed: {self.mode_feed}\n"
                f"Teach Status: {self.teach_status}\n"
                f"Motion Status: {self.motion_status}\n"
                f"Trajectory Num: {self.trajectory_num}\n"
                f"Error Code: {self._err_code}\n"
                f"Error Status: \n{self.err_status}\n")

    def __repr__(self):
        return self.__str__()


#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# 设置机械臂重置，需要在mit或者示教模式切换为位置速度控制模式时执行

from typing import (
    Optional,
)
import time
from piper_sdk import *

# 测试代码
if __name__ == "__main__":
    piper = C_PiperInterface()
    piper.ConnectPort()
    piper.MotionCtrl_1(0x02,0,0)#恢复
    piper.MotionCtrl_2(0, 0, 0, 0x00)#位置速度模式



#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# V2版本sdk
# 夹爪/示教器参数设置指令
# 第一次使用夹爪或者示教器时，需要设定一下这两个末端执行器参数，否则会出现数据没有反馈并且执行器无法控制的情况
# 一般情况下 GripperTeachingPendantParamConfig 函数第二个参数 max_range_config 是70

from piper_sdk import *

if __name__ == "__main__":
    piper = C_PiperInterface_V2("can0")
    piper.ConnectPort()
    piper.GripperTeachingPendantParamConfig(100, 70)
    


#!/usr/bin/env python3
# -*-coding:utf8-*-

class ArmMsgMotionCtrlCartesian():
    '''
    msg_v1_transmit
    
    机械臂运动控制直角坐标系指令
    
    CAN ID:
        0x152,0x153,0x154

    Args:
        X_axis: X坐标,单位0.001mm
        Y_axis: Y坐标,单位0.001mm
        Z_axis: Z坐标,单位0.001mm
        RX_axis: RX角度,单位0.001度
        RY_axis: RY角度,单位0.001度
        RZ_axis: RZ角度,单位0.001度
    '''
    '''
    msg_v1_transmit
    
    Robotic Arm Motion Control Command in Cartesian Coordinate System

    CAN ID:
        0x152, 0x153, 0x154

    Args:
        X_axis: X-axis coordinate, in 0.001 mm.
        Y_axis: Y-axis coordinate, in 0.001 mm.
        Z_axis: Z-axis coordinate, in 0.001 mm.
        RX_axis: Rotation about X-axis, in 0.001 degrees.
        RY_axis: Rotation about Y-axis, in 0.001 degrees.
        RZ_axis: Rotation about Z-axis, in 0.001 degrees.
    '''
    def __init__(self, 
                 X_axis: int = 0, 
                 Y_axis: int = 0, 
                 Z_axis: int = 0, 
                 RX_axis: int = 0, 
                 RY_axis: int = 0, 
                 RZ_axis: int = 0):
        self.X_axis = X_axis
        self.Y_axis = Y_axis
        self.Z_axis = Z_axis
        self.RX_axis = RX_axis
        self.RY_axis = RY_axis
        self.RZ_axis = RZ_axis

    def __str__(self):
        dict_ = [
            (" X_axis ", self.X_axis),
            (" Y_axis ", self.Y_axis),
            (" Z_axis ", self.Z_axis),
            (" RX_axis ", self.RX_axis),
            (" RY_axis ", self.RY_axis),
            (" RZ_axis ", self.RZ_axis)
        ]

        # 生成格式化字符串，保留三位小数
        formatted_ = "\n".join([f"{name}: {value}" for name, value in dict_])
        
        return f"ArmMsgMotionCtrlCartesian:\n{formatted_}"
    
    def __repr__(self):
        return self.__str__()


#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# 读取机械臂消息并打印,需要先安装piper_sdk
import time
from piper_sdk import *

# 测试代码
if __name__ == "__main__":
    piper = C_PiperInterface("can0")
    piper.ConnectPort(True)
    count = 0
    while True:
        count += 1
        print("---------------",count)
        if(count > 200 and count < 400):
            piper.DisconnectPort()
        elif(count > 400):
            piper.ConnectPort()
        print()
        print(f"isOK: {piper.isOk()}")
        print(f"status: {piper.GetArmStatus().Hz}")
        print(f"end_pose: {piper.GetArmEndPoseMsgs().Hz}")
        print(f"joint_states: {piper.GetArmJointMsgs().Hz}")
        print(f"gripper_msg: {piper.GetArmGripperMsgs().Hz}")
        print(f"high_spd: {piper.GetArmHighSpdInfoMsgs().Hz}")
        print(f"low_spd: {piper.GetArmLowSpdInfoMsgs().Hz}")
        print(f"joint_ctrl: {piper.GetArmJointCtrl().Hz}")
        print(f"gripper_ctrl: {piper.GetArmGripperCtrl().Hz}")
        print(f"ctrl_151: {piper.GetArmCtrlCode151().Hz}")
        print()
        time.sleep(0.01)
        pass



#!/usr/bin/env python3
# -*-coding:utf8-*-
from typing_extensions import (
    Literal,
)
class ArmMsgGripperCtrl:
    '''
    msg_v2_transmit
    
    夹爪控制指令
    
    CAN ID:
        0x159
    
    Args:
        grippers_angle: 夹爪行程
        grippers_effort: 夹爪扭矩,范围0-5000,对应0-5N/m
        status_code: 
                0x00失能;
                0x01使能;
                0x02失能清除错误;
                0x03使能清除错误.
        set_zero: 夹爪零点设置
                0x00无效值
                0xAE设置零点
    
    位描述:
    
        Byte 0 grippers_angle: int32, 单位 0.001°, 夹爪角度,以整数表示。
        Byte 1
        Byte 2
        Byte 3
        Byte 4 grippers_effort: uint16, 单位 0.001N/m, 夹爪扭矩,以整数表示。
        Byte 5
        Byte 6 status_code: uint8, 夹爪状态码, 使能/失能/清除错误;
                0x00失能;
                0x01使能;
                0x02失能清除错误;
                0x03使能清除错误.
        Byte 7 set_zero: uint8, 设定当前位置为0点
                0x00无效值
                0xAE设置零点
    '''
    '''
    msg_v2_transmit
    
    Gripper Control Command

    CAN ID:
        0x159

    Args:
        grippers_angle: Gripper stroke, represented as an integer, unit: 0.001°.
        grippers_effort: Gripper torque, represented as an integer, unit: 0.001N·m.Range 0-5000,corresponse 0-5N/m
        status_code: 
            0x00: Disable;
            0x01: Enable;
            0x03: Enable with clear error;
            0x02: Disable with clear error.
        set_zero: Set the current position as the zero point.
            0x00: Invalid;
            0xAE: Set zero.

    Bit Description:

        Byte 0-3 grippers_angle: int32, unit: 0.001°, represents the gripper angle.
        Byte 4-5 grippers_effort: uint16, unit: 0.001N·m, represents the gripper torque.
        Byte 6 status_code: uint8, gripper status code for enable/disable/clear error.
            0x00: Disable;
            0x01: Enable;
            0x03: Enable with clear error;
            0x02: Disable with clear error.
        Byte 7 set_zero: uint8, flag to set the current position as the zero point.
            0x00: Invalid;
            0xAE: Set zero.
    '''
    def __init__(self, 
                 grippers_angle: int = 0, 
                 grippers_effort: int = 0, 
                 status_code: Literal[0x00, 0x01, 0x02, 0x03] = 0,
                 set_zero: Literal[0x00, 0xAE] = 0):
        if status_code not in [0x00, 0x01, 0x02, 0x03]:
            raise ValueError(f"'status_code' Value {status_code} out of range [0x00, 0x01, 0x02, 0x03]")
        if not (0 <= grippers_effort <= 5000):
            raise ValueError(f"'grippers_effort' Value {grippers_effort} out of range 0-5000")
        if set_zero not in [0x00, 0xAE]:
            raise ValueError(f"'set_zero' Value {set_zero} out of range [0x00,0xAE]")
        self.grippers_angle = grippers_angle
        self.grippers_effort = grippers_effort
        self.status_code = status_code
        self.set_zero = set_zero

    def __str__(self):
        return (f"ArmMsgGripperCtrl(\n"
                f"  grippers_angle: {self.grippers_angle}, {self.grippers_angle * 0.001:.3f},\n"
                f"  grippers_effort: {self.grippers_effort} \t {self.grippers_effort * 0.001:.3f},\n"
                f"  status_code: {self.status_code},\n"
                f"  set_zero: {self.set_zero}\n"
                f")")

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# 设定机械臂碰撞防护等级并打印

from typing import (
    Optional,
)
import time
from piper_sdk import *

# 测试代码
if __name__ == "__main__":
    piper = C_PiperInterface("can0",False)
    piper.ConnectPort()
    # piper.CrashProtectionConfig(8,8,8,8,8,8)
    piper.CrashProtectionConfig(0,0,0,0,0,0)
    while True:
        piper.ArmParamEnquiryAndConfig(0x02, 0x00, 0x00, 0x00, 0x03)

        print(piper.GetCrashProtectionLevelFeedback())
        time.sleep(0.01)


#!/usr/bin/env python3
# -*-coding:utf8-*-

from abc import ABC, abstractmethod
import time
from enum import Enum, auto
import can
from can.message import Message

from typing import (
    Optional,
)
from .arm_msg_type import ArmMsgType
from .feedback.arm_end_pose import ArmMsgEndPoseFeedBack
from .feedback.arm_joint_feedback import ArmMsgJointFeedBack
from .feedback.arm_status import ArmMsgStatus
from .feedback.gripper_feedback import ArmMsgGripperFeedBack
from .feedback.arm_feedback_current_end_vel_acc_param import ArmMsgFeedbackCurrentEndVelAccParam
from .feedback.arm_feedback_current_motor_angle_limit_max_spd import ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd, ArmMsgFeedbackAllCurrentMotorAngleLimitMaxSpd
from .feedback.arm_feedback_current_motor_max_acc_limit import ArmMsgFeedbackCurrentMotorMaxAccLimit, ArmMsgFeedbackAllCurrentMotorMaxAccLimit
from .feedback.arm_feedback_joint_vel_acc import ArmMsgFeedbackJointVelAcc, ArmMsgFeedbackAllJointVelAcc
from .feedback.arm_high_spd_feedback import ArmHighSpdFeedback
from .feedback.arm_low_spd_feedback import ArmLowSpdFeedback
#----------------------------------基于V1.5-2版本后---------------------------------------------#
from .feedback.arm_gripper_teaching_param_feedback import ArmMsgGripperTeachingPendantParamFeedback
#---------------------------------------------------------------------------------------------#

from .transmit.arm_motion_ctrl_1 import ArmMsgMotionCtrl_1
from .transmit.arm_motion_ctrl_2 import ArmMsgMotionCtrl_2
from .transmit.arm_motion_ctrl_cartesian import ArmMsgMotionCtrlCartesian
from .transmit.arm_joint_ctrl import ArmMsgJointCtrl
from .transmit.arm_circular_pattern import ArmMsgCircularPatternCoordNumUpdateCtrl
from .transmit.arm_gripper_ctrl import ArmMsgGripperCtrl
from .transmit.arm_master_slave_config import ArmMsgMasterSlaveModeConfig
from .transmit.arm_motor_enable_disable import ArmMsgMotorEnableDisableConfig
from .transmit.arm_search_motor_max_angle_spd_acc_limit import ArmMsgSearchMotorMaxAngleSpdAccLimit
from .transmit.arm_motor_angle_limit_max_spd_config import ArmMsgMotorAngleLimitMaxSpdSet
from .transmit.arm_joint_config import ArmMsgJointConfig
from .transmit.arm_set_instruction_response import ArmMsgInstructionResponseConfig
from .transmit.arm_param_enquiry_and_config import ArmMsgParamEnquiryAndConfig
from .transmit.arm_end_vel_acc_param_config import ArmMsgEndVelAccParamConfig
from .transmit.arm_crash_protection_rating_config import ArmMsgCrashProtectionRatingConfig
from .feedback.arm_crash_protection_rating_feedback import ArmMsgCrashProtectionRatingFeedback
#----------------------------------基于V1.5-2版本后---------------------------------------------#
from .transmit.arm_joint_mit_ctrl import ArmMsgJointMitCtrl
from .transmit.arm_joint_mit_ctrl import ArmMsgAllJointMitCtrl
from .transmit.arm_gripper_teaching_param_config import ArmMsgGripperTeachingPendantParamConfig
#---------------------------------------------------------------------------------------------#
class PiperMessage:
    '''
    msg_v2
    
    Piper机械臂全部消息,为所有消息的汇总
    '''
    '''
    msg_v2
    
    Piper Robotic Arm Complete Message Summary
    '''
    def __init__(self, 
                #  反馈
                 type_: 'ArmMsgType' = None,
                 arm_status_msgs: 'ArmMsgStatus' = None,
                 arm_joint_feedback: 'ArmMsgJointFeedBack' = None,
                 gripper_feedback: 'ArmMsgGripperFeedBack' = None,
                 arm_end_pose: 'ArmMsgEndPoseFeedBack'=None,
                 arm_feedback_current_motor_angle_limit_max_spd:'ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd'=None,
                 arm_feedback_current_end_vel_acc_param:'ArmMsgFeedbackCurrentEndVelAccParam'=None,
                 arm_feedback_current_motor_max_acc_limit:'ArmMsgFeedbackCurrentMotorMaxAccLimit'=None,
                 arm_crash_protection_rating_feedback:'ArmMsgCrashProtectionRatingFeedback'=None,
                 #  arm_feedback_joint_vel_acc:'ArmMsgFeedbackJointVelAcc'=None
                #  arm_feedback_all_current_motor_angle_limit_max_spd:'ArmMsgFeedbackAllCurrentMotorAngleLimitMaxSpd'=None,
                #  arm_feedback_all_motor_max_acc_limit:'ArmMsgFeedbackAllCurrentMotorMaxAccLimit'=None,
                 arm_high_spd_feedback:'ArmHighSpdFeedback'=None,
                 arm_low_spd_feedback:'ArmLowSpdFeedback'=None,
                 arm_gripper_teaching_param_feedback:'ArmMsgGripperTeachingPendantParamFeedback'=None,
                #  发送
                 arm_motion_ctrl_1: 'ArmMsgMotionCtrl_1'=None,
                 arm_motion_ctrl_2: 'ArmMsgMotionCtrl_2'=None,
                 arm_motion_ctrl_cartesian: 'ArmMsgMotionCtrlCartesian'=None,
                 arm_joint_ctrl: 'ArmMsgJointCtrl'=None,
                 arm_circular_ctrl: 'ArmMsgCircularPatternCoordNumUpdateCtrl'=None,
                 arm_gripper_ctrl: 'ArmMsgGripperCtrl'=None,
                 arm_joint_mit_ctrl: 'ArmMsgJointMitCtrl'=None,
                 arm_ms_config: 'ArmMsgMasterSlaveModeConfig'=None,
                 arm_motor_enable: 'ArmMsgMotorEnableDisableConfig'=None,
                 arm_search_motor_max_angle_spd_acc_limit:'ArmMsgSearchMotorMaxAngleSpdAccLimit'=None,
                 arm_motor_angle_limit_max_spd_set:'ArmMsgMotorAngleLimitMaxSpdSet'=None,
                 arm_joint_config:'ArmMsgJointConfig'=None,
                 arm_set_instruction_response:'ArmMsgInstructionResponseConfig'=None,
                 arm_param_enquiry_and_config:'ArmMsgParamEnquiryAndConfig'=None,
                 arm_end_vel_acc_param_config:'ArmMsgEndVelAccParamConfig'=None,
                 arm_crash_protection_rating_config:'ArmMsgCrashProtectionRatingConfig'=None,
                 arm_gripper_teaching_param_config:'ArmMsgGripperTeachingPendantParamConfig'=None
                 ):
        #-------------------------------反馈-------------------------------------------
        # 初始化数据帧类型
        self.type_ = type_
        # 初始化机械臂状态消息
        self.arm_status_msgs = arm_status_msgs if arm_status_msgs else ArmMsgStatus()
        # 初始化机械臂关节反馈
        self.arm_joint_feedback = arm_joint_feedback if arm_joint_feedback else ArmMsgJointFeedBack()
        # 初始化夹爪反馈
        self.gripper_feedback = gripper_feedback if gripper_feedback else ArmMsgGripperFeedBack()
        # 初始化末端姿态反馈
        self.arm_end_pose = arm_end_pose if arm_end_pose else ArmMsgEndPoseFeedBack()
        # 驱动器信息高速反馈
        self.arm_high_spd_feedback_1 = arm_high_spd_feedback if arm_high_spd_feedback else ArmHighSpdFeedback()
        self.arm_high_spd_feedback_2 = arm_high_spd_feedback if arm_high_spd_feedback else ArmHighSpdFeedback()
        self.arm_high_spd_feedback_3 = arm_high_spd_feedback if arm_high_spd_feedback else ArmHighSpdFeedback()
        self.arm_high_spd_feedback_4 = arm_high_spd_feedback if arm_high_spd_feedback else ArmHighSpdFeedback()
        self.arm_high_spd_feedback_5 = arm_high_spd_feedback if arm_high_spd_feedback else ArmHighSpdFeedback()
        self.arm_high_spd_feedback_6 = arm_high_spd_feedback if arm_high_spd_feedback else ArmHighSpdFeedback()
        # 驱动器信息低速反馈
        self.arm_low_spd_feedback_1 = arm_low_spd_feedback if arm_low_spd_feedback else ArmLowSpdFeedback()
        self.arm_low_spd_feedback_2 = arm_low_spd_feedback if arm_low_spd_feedback else ArmLowSpdFeedback()
        self.arm_low_spd_feedback_3 = arm_low_spd_feedback if arm_low_spd_feedback else ArmLowSpdFeedback()
        self.arm_low_spd_feedback_4 = arm_low_spd_feedback if arm_low_spd_feedback else ArmLowSpdFeedback()
        self.arm_low_spd_feedback_5 = arm_low_spd_feedback if arm_low_spd_feedback else ArmLowSpdFeedback()
        self.arm_low_spd_feedback_6 = arm_low_spd_feedback if arm_low_spd_feedback else ArmLowSpdFeedback()
        # 夹爪/示教器参数反馈指令
        self.arm_gripper_teaching_param_feedback = arm_gripper_teaching_param_feedback \
            if arm_gripper_teaching_param_feedback else ArmMsgGripperTeachingPendantParamFeedback()
        #-------------------------------发送-------------------------------------------
        self.arm_motion_ctrl_1 = arm_motion_ctrl_1 if arm_motion_ctrl_1 else ArmMsgMotionCtrl_1()
        self.arm_motion_ctrl_2 = arm_motion_ctrl_2 if arm_motion_ctrl_2 else ArmMsgMotionCtrl_2()
        self.arm_motion_ctrl_cartesian = arm_motion_ctrl_cartesian if arm_motion_ctrl_cartesian else ArmMsgMotionCtrlCartesian()
        self.arm_joint_ctrl = arm_joint_ctrl if arm_joint_ctrl else ArmMsgJointCtrl()
        self.arm_circular_ctrl = arm_circular_ctrl \
            if arm_circular_ctrl else ArmMsgCircularPatternCoordNumUpdateCtrl()
        # 夹爪控制
        self.arm_gripper_ctrl = arm_gripper_ctrl if arm_gripper_ctrl else ArmMsgGripperCtrl()
        # 关节mit控制
        self.arm_joint_mit_ctrl = arm_joint_mit_ctrl if arm_joint_mit_ctrl else ArmMsgJointMitCtrl()
        self.arm_ms_config = arm_ms_config if arm_ms_config else ArmMsgMasterSlaveModeConfig()
        # 电机使能/失能设置指令
        self.arm_motor_enable = arm_motor_enable if arm_motor_enable else ArmMsgMotorEnableDisableConfig()
        # 查询电机角度/最大速度/最大加速度限制指令
        self.arm_search_motor_max_angle_spd_acc_limit = arm_search_motor_max_angle_spd_acc_limit \
            if arm_search_motor_max_angle_spd_acc_limit else ArmMsgSearchMotorMaxAngleSpdAccLimit()
        # 反馈当前电机限制角度/最大速度
        self.arm_feedback_current_motor_angle_limit_max_spd = arm_feedback_current_motor_angle_limit_max_spd \
            if arm_feedback_current_motor_angle_limit_max_spd else ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd()
        # 电机角度限制/最大速度设置指令
        self.arm_motor_angle_limit_max_spd_set = arm_motor_angle_limit_max_spd_set \
            if arm_motor_angle_limit_max_spd_set else ArmMsgMotorAngleLimitMaxSpdSet()
        # 关节设置指令
        self.arm_joint_config = arm_joint_config \
            if arm_joint_config else ArmMsgJointConfig()
        # 设置指令应答
        self.arm_set_instruction_response = arm_set_instruction_response \
            if arm_set_instruction_response else ArmMsgInstructionResponseConfig()
        # 机械臂参数查询与设置指令
        self.arm_param_enquiry_and_config = arm_param_enquiry_and_config \
            if arm_param_enquiry_and_config else ArmMsgParamEnquiryAndConfig()
        # 反馈当前末端速度/加速度参数
        self.arm_feedback_current_end_vel_acc_param = arm_feedback_current_end_vel_acc_param \
            if arm_feedback_current_end_vel_acc_param else ArmMsgFeedbackCurrentEndVelAccParam()
        # 末端速度/加速度参数设置指令
        self.arm_end_vel_acc_param_config = arm_end_vel_acc_param_config \
            if arm_end_vel_acc_param_config else ArmMsgEndVelAccParamConfig()
        # 碰撞防护等级设置指令
        self.arm_crash_protection_rating_config = arm_crash_protection_rating_config \
            if arm_crash_protection_rating_config else ArmMsgCrashProtectionRatingConfig()
        # 碰撞防护等级设置反馈指令
        self.arm_crash_protection_rating_feedback = arm_crash_protection_rating_feedback \
            if arm_crash_protection_rating_feedback else ArmMsgCrashProtectionRatingFeedback()
        # 反馈当前电机最大加速度限制
        self.arm_feedback_current_motor_max_acc_limit = arm_feedback_current_motor_max_acc_limit \
            if arm_feedback_current_motor_max_acc_limit else ArmMsgFeedbackCurrentMotorMaxAccLimit()
        # 夹爪/示教器参数设置指令
        self.arm_gripper_teaching_param_config = arm_gripper_teaching_param_config \
            if arm_gripper_teaching_param_config else ArmMsgGripperTeachingPendantParamConfig()
        # 反馈各个关节当前末端速度/加速度
        # self.arm_feedback_joint_vel_acc = arm_feedback_joint_vel_acc \
        #     if arm_feedback_joint_vel_acc else ArmMsgFeedbackJointVelAcc()
        # 全部的电机当前限制角度/最大速度
        # self.arm_feedback_all_current_motor_angle_limit_max_spd = arm_feedback_all_current_motor_angle_limit_max_spd \
        #     if arm_feedback_all_current_motor_angle_limit_max_spd else ArmMsgFeedbackAllCurrentMotorAngleLimitMaxSpd()
        # # 全部的电机最大加速度限制
        # self.arm_feedback_all_motor_max_acc_limit = arm_feedback_all_motor_max_acc_limit \
        #     if arm_feedback_all_motor_max_acc_limit else ArmMsgFeedbackAllCurrentMotorMaxAccLimit()
        self.firmware_data = bytearray()

    def __str__(self):
        # feedback
        if(self.type_ == ArmMsgType.PiperMsgStatusFeedback):
            return (f"Type: {self.type_}\n"f"Arm Status: {self.arm_status_msgs}\n")
        elif(self.type_ == ArmMsgType.PiperMsgJointFeedBack_12):
            return (f"Type: {self.type_}\n"f"Joint Feed: {self.arm_joint_feedback}\n")
        elif(self.type_ == ArmMsgType.PiperMsgJointFeedBack_34):
            return (f"Type: {self.type_}\n"f"Joint Feed: {self.arm_joint_feedback}\n")
        elif(self.type_ == ArmMsgType.PiperMsgJointFeedBack_56):
            return (f"Type: {self.type_}\n"f"Joint Feed: {self.arm_joint_feedback}\n")
        elif(self.type_ == ArmMsgType.PiperMsgGripperFeedBack):
            return (f"Type: {self.type_}\n"f"Gripper Feed: {self.gripper_feedback}\n")
        elif(self.type_ == ArmMsgType.PiperMsgEndPoseFeedback_1):
            return (f"Type: {self.type_}\n"f"End Pose Feed: {self.arm_end_pose}\n")
        elif(self.type_ == ArmMsgType.PiperMsgEndPoseFeedback_2):
            return (f"Type: {self.type_}\n"f"End Pose Feed: {self.arm_end_pose}\n")
        elif(self.type_ == ArmMsgType.PiperMsgEndPoseFeedback_3):
            return (f"Type: {self.type_}\n"f"End Pose Feed: {self.arm_end_pose}\n")
        elif(self.type_ == ArmMsgType.PiperMsgHighSpdFeed_1):
            return (f"Type: {self.type_}\n"f"High Spd Feedback: {self.arm_high_spd_feedback_1}\n")
        elif(self.type_ == ArmMsgType.PiperMsgHighSpdFeed_2):
            return (f"Type: {self.type_}\n"f"High Spd Feedback: {self.arm_high_spd_feedback_2}\n")
        elif(self.type_ == ArmMsgType.PiperMsgHighSpdFeed_3):
            return (f"Type: {self.type_}\n"f"High Spd Feedback: {self.arm_high_spd_feedback_3}\n")
        elif(self.type_ == ArmMsgType.PiperMsgHighSpdFeed_4):
            return (f"Type: {self.type_}\n"f"High Spd Feedback: {self.arm_high_spd_feedback_4}\n")
        elif(self.type_ == ArmMsgType.PiperMsgHighSpdFeed_5):
            return (f"Type: {self.type_}\n"f"High Spd Feedback: {self.arm_high_spd_feedback_5}\n")
        elif(self.type_ == ArmMsgType.PiperMsgHighSpdFeed_6):
            return (f"Type: {self.type_}\n"f"High Spd Feedback: {self.arm_high_spd_feedback_6}\n")
        elif(self.type_ == ArmMsgType.PiperMsgLowSpdFeed_1):
            return (f"Type: {self.type_}\n"f"Low Spd Feedback: {self.arm_low_spd_feedback_1}\n")
        elif(self.type_ == ArmMsgType.PiperMsgLowSpdFeed_2):
            return (f"Type: {self.type_}\n"f"Low Spd Feedback: {self.arm_low_spd_feedback_2}\n")
        elif(self.type_ == ArmMsgType.PiperMsgLowSpdFeed_3):
            return (f"Type: {self.type_}\n"f"Low Spd Feedback: {self.arm_low_spd_feedback_3}\n")
        elif(self.type_ == ArmMsgType.PiperMsgLowSpdFeed_4):
            return (f"Type: {self.type_}\n"f"Low Spd Feedback: {self.arm_low_spd_feedback_4}\n")
        elif(self.type_ == ArmMsgType.PiperMsgLowSpdFeed_5):
            return (f"Type: {self.type_}\n"f"Low Spd Feedback: {self.arm_low_spd_feedback_5}\n")
        elif(self.type_ == ArmMsgType.PiperMsgLowSpdFeed_6):
            return (f"Type: {self.type_}\n"f"Low Spd Feedback: {self.arm_low_spd_feedback_6}\n")
        # 发送,transmit
        elif(self.type_ == ArmMsgType.PiperMsgMotionCtrl_1):
            return (f"Type: {self.type_}\n"f"PiperMsgMotionCtrl_1: {self.arm_motion_ctrl_1}\n")
        elif(self.type_ == ArmMsgType.PiperMsgMotionCtrl_2):
            return (f"Type: {self.type_}\n"f"PiperMsgMotionCtrl_2: {self.arm_motion_ctrl_2}\n")
        elif(self.type_ == ArmMsgType.PiperMsgMotionCtrlCartesian_1):
            return (f"Type: {self.type_}\n"f"ArmMsgMotionCtrlCartesian: {self.arm_motion_ctrl_cartesian}\n")
        elif(self.type_ == ArmMsgType.PiperMsgMotionCtrlCartesian_2):
            return (f"Type: {self.type_}\n"f"ArmMsgMotionCtrlCartesian: {self.arm_motion_ctrl_cartesian}\n")
        elif(self.type_ == ArmMsgType.PiperMsgMotionCtrlCartesian_3):
            return (f"Type: {self.type_}\n"f"ArmMsgMotionCtrlCartesian: {self.arm_motion_ctrl_cartesian}\n")
        elif(self.type_ == ArmMsgType.PiperMsgJointCtrl_12):
            return (f"Type: {self.type_}\n"f"ArmMsgJointCtrl: {self.arm_joint_ctrl}\n")
        elif(self.type_ == ArmMsgType.PiperMsgJointCtrl_34):
            return (f"Type: {self.type_}\n"f"ArmMsgJointCtrl: {self.arm_joint_ctrl}\n")
        elif(self.type_ == ArmMsgType.PiperMsgJointCtrl_56):
            return (f"Type: {self.type_}\n"f"ArmMsgJointCtrl: {self.arm_joint_ctrl}\n")
        elif(self.type_ == ArmMsgType.PiperMsgGripperCtrl):
            return (f"Type: {self.type_}\n"f"PiperMsgGripperCtrl: {self.arm_gripper_ctrl}\n")
        else:
            return (f"Type: {self.type_}\n")

    def __repr__(self):
        return self.__str__()


#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# 读取机械臂消息并打印,需要先安装piper_sdk
from typing import (
    Optional,
)
from piper_sdk import *

# 测试代码
if __name__ == "__main__":
    piper = C_PiperInterface_V2()
    piper.ConnectPort()
    while True:
        import time
        print(piper.GetArmStatus())
        time.sleep(0.005)
        pass



#!/usr/bin/env python3
# -*-coding:utf8-*-

class ArmMsgJointFeedBack():
    '''
    msg_v2_feedback
    
    机械臂关节角度反馈,单位0.001度
    
    CAN ID: 
        0x2A5、0x2A6、0x2A7
    
    Args:
        joint_1: 关节1反馈角度
        joint_2: 关节2反馈角度
        joint_3: 关节3反馈角度
        joint_4: 关节4反馈角度
        joint_5: 关节5反馈角度
        joint_6: 关节6反馈角度
    '''
    '''
    msg_v2_feedback
    
    Joint Angle Feedback for Robotic Arm, in 0.001 Degrees
    
    CAN ID: 
        0x2A5、0x2A6、0x2A7
    
    Args:
        joint_1: Feedback angle of joint 1, in 0.001 degrees.
        joint_2: Feedback angle of joint 2, in 0.001 degrees.
        joint_3: Feedback angle of joint 3, in 0.001 degrees.
        joint_4: Feedback angle of joint 4, in 0.001 degrees.
        joint_5: Feedback angle of joint 5, in 0.001 degrees.
        joint_6: Feedback angle of joint 6, in 0.001 degrees.
    '''
    def __init__(self, 
                 joint_1: int = 0, 
                 joint_2: int = 0, 
                 joint_3: int = 0, 
                 joint_4: int = 0, 
                 joint_5: int = 0, 
                 joint_6: int = 0):
        self.joint_1 = joint_1
        self.joint_2 = joint_2
        self.joint_3 = joint_3
        self.joint_4 = joint_4
        self.joint_5 = joint_5
        self.joint_6 = joint_6

    def __str__(self):
        # 将角度乘以0.001，并保留三位小数
        joint_angles = [
            ("Joint 1", self.joint_1, self.joint_1 * 0.001),
            ("Joint 2", self.joint_2, self.joint_2 * 0.001),
            ("Joint 3", self.joint_3, self.joint_3 * 0.001),
            ("Joint 4", self.joint_4, self.joint_4 * 0.001),
            ("Joint 5", self.joint_5, self.joint_5 * 0.001),
            ("Joint 6", self.joint_6, self.joint_6 * 0.001)
        ]

        # 生成格式化字符串，保留三位小数
        formatted_angles = "\n".join([f"{name}:{angle}, {angle_f:.3f}" for name, angle, angle_f in joint_angles])
        
        return f"ArmMsgJointFeedBack:\n{formatted_angles}"
    
    def __repr__(self):
        return self.__str__()


#!/usr/bin/env python3
# -*-coding:utf8-*-
from typing_extensions import (
    Literal,
)
class ArmMsgMotorAngleLimitMaxSpdSet:
    '''
    msg_v2_transmit
    
    电机角度限制/最大速度设置指令-V2(基于V1.5-2版本后增加无效数值0x7FFF)

    CAN ID:
        0x474

    Args:
        motor_num: 关节电机序号
        max_angle_limit: 最大角度限制,单位 0.1°,0x7FFF为设定无效数值
        min_angle_limit: 最小角度限制,单位 0.1°,0x7FFF为设定无效数值
        max_joint_spd: 最大关节速度,单位 0.001rad/s,范围[0,3000],0x7FFF为设定无效数值
    
    |joint_name|     limit(rad)     |    limit(angle)    |     limit(rad/s)   |
    |----------|     ----------     |     ----------     |     ----------     |
    |joint1    |   [-2.618, 2.618]  |    [-150.0, 150.0] |      [0, 3.0]      |
    |joint2    |   [0, 3.14]        |    [0, 180.0]      |      [0, 3.0]      |
    |joint3    |   [-2.967, 0]      |    [-170, 0]       |      [0, 3.0]      |
    |joint4    |   [-1.745, 1.745]  |    [-100.0, 100.0] |      [0, 3.0]      |
    |joint5    |   [-1.22, 1.22]    |    [-70.0, 70.0]   |      [0, 3.0]      |
    |joint6    |   [-2.0944, 2.0944]|    [-120.0, 120.0] |      [0, 3.0]      |
    
    位描述:
    
        Byte 0: 关节电机序号 uint8, 值域 1-6:1-6 代表关节驱动器序号
        Byte 1: 最大角度限制 H: int16, 单位 0.1°(基于V1.5-2版本后增加无效数值0x7FFF)
        Byte 2: 最大角度限制 L
        Byte 3: 最小角度限制 H: int16, 单位 0.1°(基于V1.5-2版本后增加无效数值0x7FFF)
        Byte 4: 最小角度限制 L
        Byte 5: 最大关节速度 H: uint16, 单位 0.001rad/s(基于V1.5-2版本后增加无效数值0x7FFF)
        Byte 6: 最大关节速度 L
    '''
    '''
    msg_v2_transmit
    
    Motor Angle Limits/Maximum Speed Setting Command-V2(Based on version V1.5-2 and later, the invalid value 0x7FFF is added.)

    CAN ID:
        0x474

    Args:
        motor_num: Joint motor index.
        max_angle_limit: Maximum angle limit, unit 0.1°,0x7FFF is defined as the invalid value.
        min_angle_limit: Minimum angle limit, unit 0.1°,0x7FFF is defined as the invalid value.
        max_joint_spd: Maximum joint speed, unit 0.001 rad/s,Range [0, 3000],0x7FFF is defined as the invalid value.

    |joint_name|     limit(rad)     |    limit(angle)    |     limit(rad/s)   |
    |----------|     ----------     |     ----------     |     ----------     |
    |joint1    |   [-2.618, 2.618]  |    [-150.0, 150.0] |      [0, 3.0]      |
    |joint2    |   [0, 3.14]        |    [0, 180.0]      |      [0, 3.0]      |
    |joint3    |   [-2.967, 0]      |    [-170, 0]       |      [0, 3.0]      |
    |joint4    |   [-1.745, 1.745]  |    [-100.0, 100.0] |      [0, 3.0]      |
    |joint5    |   [-1.22, 1.22]    |    [-70.0, 70.0]   |      [0, 3.0]      |
    |joint6    |   [-2.0944, 2.0944]|    [-120.0, 120.0] |      [0, 3.0]      |
    
    Bit Description:

        Byte 0: Joint motor index, uint8, range 1-6.
        Byte 1: Maximum angle limit (high byte), int16, unit 0.1°.(Based on version V1.5-2 and later, the invalid value 0x7FFF is added.)
        Byte 2: Maximum angle limit (low byte).
        Byte 3: Minimum angle limit (high byte), int16, unit 0.1°.(Based on version V1.5-2 and later, the invalid value 0x7FFF is added.)
        Byte 4: Minimum angle limit (low byte).
        Byte 5: Maximum joint speed (high byte), uint16, unit 0.001 rad/s.(Based on version V1.5-2 and later, the invalid value 0x7FFF is added.)
        Byte 6: Maximum joint speed (low byte).
    '''
    def __init__(self, 
                 motor_num: Literal[1, 2, 3, 4, 5, 6] = 1, 
                 max_angle_limit: int = 0x7FFF, 
                 min_angle_limit: int = 0x7FFF,
                 max_joint_spd: int = 0x7FFF):
        if motor_num not in [1, 2, 3, 4, 5, 6]:
            raise ValueError(f"'motor_num' Value {motor_num} out of range [1, 2, 3, 4, 5, 6]")
        if not (0 <= max_joint_spd <= 3000 or max_joint_spd == 0x7FFF):
            raise ValueError(f"'max_joint_spd' Value {max_joint_spd} out of range 0-3000 or not equal to 0x7FFF")
        self.motor_num = motor_num
        self.max_angle_limit = max_angle_limit
        self.min_angle_limit = min_angle_limit
        self.max_joint_spd = max_joint_spd

    def __str__(self):
        return (f"ArmMsgMotorAngleSpdLimitConfig(\n"
                f"  motor_num: {self.motor_num},\n"
                f"  max_angle_limit: {self.max_angle_limit}, {self.max_angle_limit * 0.1:.1f},\n"
                f"  min_angle_limit: {self.min_angle_limit}, {self.min_angle_limit * 0.1:.1f},\n"
                f"  max_joint_spd: {self.max_joint_spd}, {self.max_joint_spd * 0.3:.3f}\n"
                f")")

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-
# 当有多个机械臂的时候，可以创建多实例，通过识别can_port避免创建读取相同can端口的实例
# 注意需要多个can模块
from piper_sdk import *

# 测试代码
if __name__ == "__main__":
    piper = C_PiperInterface("can0")
    piper.ConnectPort(True)
    piper = C_PiperInterface("can0",can_auto_init=False)
    piper.ConnectPort(True)
    count = 0
    piper = C_PiperInterface("can1",can_auto_init=False)
    piper.ConnectPort()
    piper = C_PiperInterface("can1")
    piper.ConnectPort()



import math
from typing_extensions import (
    Literal,
)

class C_PiperForwardKinematics():
    def __init__(self, dh_is_offset: Literal[0x00, 0x01] = 0x00):
        self.RADIAN = 180 / math.pi
        self.PI = math.pi
        # Denavit-Hartenberg parameters for each link
        # _a: link lengths
        # _alpha: link twists
        # _theta: joint angles
        # _d: link offsets
        self._a     = [0     , 0                      , 285.03                   , -21.98        , 0             , 0          ]
        self._alpha = [0     , -self.PI / 2           , 0                        , self.PI / 2   , -self.PI / 2  , self.PI / 2]
        self._theta = [0     , -self.PI * 174.22 / 180, -100.78 / 180 * self.PI  , 0             , 0             , 0          ]
        self._d     = [123   , 0                      , 0                        , 250.75        , 0             , 91         ]
        self.init_pos   = [55.0  , 0.0                    , 205.0                    , 0.0           , 85.0          , 0.0] # unit xyz-mm, rpy-degree
        # if j2, j3 offset 2°
        if(dh_is_offset == 0x01):
            self._a     = [0     , 0                      , 285.03                   , -21.98        , 0             , 0          ]
            self._alpha = [0     , -self.PI / 2           , 0                        , self.PI / 2   , -self.PI / 2  , self.PI / 2]
            self._theta = [0     , -self.PI * 172.22 / 180, -102.78 / 180 * self.PI  , 0             , 0             , 0          ]
            self._d     = [123   , 0                      , 0                        , 250.75        , 0             , 91         ]
            self.init_pos   = [56.128, 0.0                    , 213.266                  , 0.0           , 85.0          , 0.0] # unit xyz-mm, rpy-degree
    def __MatrixToeula(self, T):
        '''
        Convert a transformation matrix to Euler angles (roll, pitch, yaw).
        T: 4x4 transformation matrix
        '''
        Pos = [0.0] * 6
        # Extract position (x, y, z)
        Pos[0] = T[3]  # x position
        Pos[1] = T[7]  # y position
        Pos[2] = T[11] # z position
        # Calculate Euler angles (roll, pitch, yaw) based on rotation matrix
        if T[8] < -1 + 0.0001:
            Pos[4] = self.PI / 2 * self.RADIAN  # pitch (beta)
            Pos[5] = 0
            Pos[3] = math.atan2(T[1], T[5]) * self.RADIAN # roll (alpha)
        elif T[8] > 1 - 0.0001:
            Pos[4] = -self.PI / 2 * self.RADIAN # pitch (beta)
            Pos[5] = 0
            Pos[3] = -math.atan2(T[1], T[5]) * self.RADIAN # roll (alpha)
        else:
            # General case for Euler angles computation
            _bt = math.atan2(-T[8], math.sqrt(T[0] * T[0] + T[4] * T[4])) # pitch (beta)
            Pos[4] = _bt * self.RADIAN
            Pos[5] = math.atan2(T[4] / math.cos(_bt), T[0] / math.cos(_bt)) * self.RADIAN # yaw (gamma)
            Pos[3] = math.atan2(T[9] / math.cos(_bt), T[10] / math.cos(_bt)) * self.RADIAN # roll (alpha)

        return Pos

    def __MatMultiply(self, matrix1, matrix2, m, l, n):
        '''
        Multiply two matrices
        matrix1: first matrix
        matrix2: second matrix
        m: number of rows in matrix1
        l: number of columns in matrix1 (rows in matrix2)
        n: number of columns in matrix2
        '''
        matrixOut = [0.0] * (m * n)
        for i in range(m):
            for j in range(n):
                tmp = 0.0
                for k in range(l):
                    tmp += matrix1[l * i + k] * matrix2[n * k + j]
                matrixOut[n * i + j] = tmp
        return matrixOut
    
    def __LinkTransformtion(self, alpha, a, theta, d):
        '''
        Compute the transformation matrix for a single link using the Denavit-Hartenberg parameters
        alpha: link twist
        a: link length
        theta: joint angle
        d: link offset
        '''
        # Precompute trigonometric functions for efficiency
        calpha = math.cos(alpha)
        salpha = math.sin(alpha)
        ctheta = math.cos(theta)
        stheta = math.sin(theta)

        T = [0.0] * 16 # 4x4 transformation matrix
        T[0] = ctheta
        T[1] = -stheta
        T[2] = 0
        T[3] = a

        T[4] = stheta * calpha
        T[5] = ctheta * calpha
        T[6] = -salpha
        T[7] = -salpha * d

        T[8] = stheta * salpha
        T[9] = ctheta * salpha
        T[10] = calpha
        T[11] = calpha * d

        T[12] = 0
        T[13] = 0
        T[14] = 0
        T[15] = 1

        return T
    
    def CalFK(self, cur_j):
        '''
        Calculate Forward Kinematics for a given joint configuration
        cur_j: list of joint angles
        Returns the positions and Euler angles for each link
        '''
        # Initialize transformation matrices
        _Rt = [[0.0] * 16 for _ in range(6)]

        # Compute the individual transformation matrices
        for i in range(6):
            c_theta = cur_j[i] + self._theta[i]
            _Rt[i] = self.__LinkTransformtion(self._alpha[i], self._a[i], c_theta, self._d[i])

        # Multiply transformation matrices
        R02 = self.__MatMultiply(_Rt[0], _Rt[1], 4, 4, 4)
        R03 = self.__MatMultiply(R02, _Rt[2], 4, 4, 4)
        R04 = self.__MatMultiply(R03, _Rt[3], 4, 4, 4)
        R05 = self.__MatMultiply(R04, _Rt[4], 4, 4, 4)
        R06 = self.__MatMultiply(R05, _Rt[5], 4, 4, 4)

        # Extract Euler angles for each transformation
        j_pos = []
        j_pos.append(self.__MatrixToeula(_Rt[0]))   # Euler angles for link1
        j_pos.append(self.__MatrixToeula(R02))      # Euler angles for link2
        j_pos.append(self.__MatrixToeula(R03))      # Euler angles for link3
        j_pos.append(self.__MatrixToeula(R04))      # Euler angles for link4
        j_pos.append(self.__MatrixToeula(R05))      # Euler angles for link5
        j_pos.append(self.__MatrixToeula(R06))      # Euler angles for link6

        return j_pos



#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
from typing import (
    Optional,
)
import time
from piper_sdk import *

def enable_fun(piper:C_PiperInterface_V2):
    '''
    使能机械臂并检测使能状态,尝试5s,如果使能超时则退出程序
    '''
    enable_flag = False
    # 设置超时时间（秒）
    timeout = 5
    # 记录进入循环前的时间
    start_time = time.time()
    elapsed_time_flag = False
    while not (enable_flag):
        elapsed_time = time.time() - start_time
        print("--------------------")
        enable_flag = piper.GetArmLowSpdInfoMsgs().motor_1.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_2.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_3.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_4.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_5.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_6.foc_status.driver_enable_status
        print("使能状态:",enable_flag)
        piper.EnableArm(7)
        piper.GripperCtrl(0,1000,0x01, 0)
        print("--------------------")
        # 检查是否超过超时时间
        if elapsed_time > timeout:
            print("超时....")
            elapsed_time_flag = True
            enable_flag = True
            break
        time.sleep(1)
        pass
    if(elapsed_time_flag):
        print("程序自动使能超时,退出程序")
        exit(0)

if __name__ == "__main__":
    piper = C_PiperInterface_V2("can0")
    piper.ConnectPort()
    piper.EnableArm(7)
    enable_fun(piper=piper)
    piper.GripperCtrl(0,1000,0x01, 0)
    factor = 1000
    position = [
                55.0, \
                0.0, \
                206.0, \
                0, \
                85.0, \
                0, \
                0]
    # position = [0.0, \
    #             0.0, \
    #             80.0, \
    #             0, \
    #             203.386, \
    #             0, \
    #             0.8]
    count = 0
    while True:
        print(piper.GetArmEndPoseMsgs())
        # print(piper.GetArmStatus())
        import time
        count  = count + 1
        # print(count)
        if(count == 0):
            print("1-----------")
            position = [
                55.0, \
                0.0, \
                206.0, \
                0, \
                85.0, \
                0, \
                0]
        elif(count == 200):
            print("2-----------")
            position = [
                55.0, \
                0.0, \
                260.0, \
                0, \
                85.0, \
                0, \
                0]
        elif(count == 400):
            print("1-----------")
            position = [
                55.0, \
                0.0, \
                206.0, \
                0, \
                85.0, \
                0, \
                0]
            count = 0
        
        X = round(position[0]*factor)
        Y = round(position[1]*factor)
        Z = round(position[2]*factor)
        RX = round(position[3]*factor)
        RY = round(position[4]*factor)
        RZ = round(position[5]*factor)
        joint_6 = round(position[6]*factor)
        print(X,Y,Z,RX,RY,RZ)
        # piper.MotionCtrl_1()
        piper.MotionCtrl_2(0x01, 0x00, 100, 0x00)
        piper.EndPoseCtrl(X,Y,Z,RX,RY,RZ)
        piper.GripperCtrl(abs(joint_6), 1000, 0x01, 0)
        time.sleep(0.01)
        pass


#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# 读取机械臂软件固件版本并打印
from typing import (
    Optional,
)
import time
from piper_sdk import *

# 测试代码
if __name__ == "__main__":
    piper = C_PiperInterface()
    piper.ConnectPort()
    time.sleep(0.025) # 需要时间去读取固件反馈帧，否则会反馈-0x4AF
    print(piper.GetPiperFirmwareVersion())
    while True:
        piper.SearchPiperFirmwareVersion()
        time.sleep(0.025)
        print(piper.GetPiperFirmwareVersion())
        



# piper_msgs/msg_v1/__init__.py

# msg_v1/__init__.py

from .arm_messages import PiperMessage
from .can_id import CanIDPiper
from .arm_msg_type import ArmMsgType
from .arm_id_type_map import ArmMessageMapping
# 导入 feedback 子模块的类
from .feedback.arm_end_pose import ArmMsgEndPoseFeedBack
from .feedback.arm_joint_feedback import ArmMsgJointFeedBack
from .feedback.arm_status import ArmMsgStatus
from .feedback.gripper_feedback import ArmMsgGripperFeedBack
from .feedback.arm_feedback_current_motor_angle_limit_max_spd import ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd, ArmMsgFeedbackAllCurrentMotorAngleLimitMaxSpd
from .feedback.arm_feedback_current_end_vel_acc_param import ArmMsgFeedbackCurrentEndVelAccParam
from .feedback.arm_feedback_current_motor_max_acc_limit import ArmMsgFeedbackCurrentMotorMaxAccLimit, ArmMsgFeedbackAllCurrentMotorMaxAccLimit
from .feedback.arm_feedback_joint_vel_acc import ArmMsgFeedbackJointVelAcc, ArmMsgFeedbackAllJointVelAcc
from .feedback.arm_crash_protection_rating_feedback import ArmMsgCrashProtectionRatingFeedback
from .feedback.arm_high_spd_feedback import ArmHighSpdFeedback
from .feedback.arm_low_spd_feedback import ArmLowSpdFeedback

# 导入 transmit 子模块（假设 transmit 中也有需要导出的类）
from .transmit.arm_motion_ctrl_1 import ArmMsgMotionCtrl_1
from .transmit.arm_motion_ctrl_2 import ArmMsgMotionCtrl_2
from .transmit.arm_motion_ctrl_cartesian import ArmMsgMotionCtrlCartesian
from .transmit.arm_joint_ctrl import ArmMsgJointCtrl
from .transmit.arm_circular_pattern import ArmMsgCircularPatternCoordNumUpdateCtrl
from .transmit.arm_gripper_ctrl import ArmMsgGripperCtrl
from .transmit.arm_master_slave_config import ArmMsgMasterSlaveModeConfig
from .transmit.arm_motor_enable_disable import ArmMsgMotorEnableDisableConfig
from .transmit.arm_search_motor_max_angle_spd_acc_limit import ArmMsgSearchMotorMaxAngleSpdAccLimit
from .transmit.arm_motor_angle_limit_max_spd_config import ArmMsgMotorAngleLimitMaxSpdSet
from .transmit.arm_joint_config import ArmMsgJointConfig
from .transmit.arm_set_instruction_response import ArmMsgInstructionResponseConfig
from .transmit.arm_param_enquiry_and_config import ArmMsgParamEnquiryAndConfig
from .transmit.arm_end_vel_acc_param_config import ArmMsgEndVelAccParamConfig
from .transmit.arm_crash_protection_rating_config import ArmMsgCrashProtectionRatingConfig

__all__ = [
    # 反馈
    'PiperMessage',
    'CanIDPiper',
    'ArmMsgEndPoseFeedBack',
    'ArmMsgJointFeedBack',
    'ArmMsgType',
    'ArmMsgStatus',
    'ArmMsgGripperFeedBack',
    'ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd',
    'ArmMsgFeedbackCurrentEndVelAccParam',
    'ArmMsgFeedbackCurrentMotorMaxAccLimit',
    'ArmMsgFeedbackJointVelAcc',
    'ArmMsgFeedbackAllCurrentMotorAngleLimitMaxSpd',
    'ArmMsgFeedbackAllCurrentMotorMaxAccLimit',
    'ArmMsgFeedbackAllJointVelAcc',
    'ArmMsgCrashProtectionRatingFeedback',
    'ArmHighSpdFeedback',
    'ArmLowSpdFeedback',
    # 发送
    'ArmMsgMotionCtrl_1',
    'ArmMsgMotionCtrl_2',
    'ArmMsgMotionCtrlCartesian',
    'ArmMsgJointCtrl',
    'ArmMsgCircularPatternCoordNumUpdateCtrl',
    'ArmMsgGripperCtrl',
    'ArmMsgMasterSlaveModeConfig',
    'ArmMsgMotorEnableDisableConfig',
    'ArmMsgSearchMotorMaxAngleSpdAccLimit',
    'ArmMsgMotorAngleLimitMaxSpdSet',
    'ArmMsgJointConfig',
    'ArmMsgInstructionResponseConfig',
    'ArmMsgParamEnquiryAndConfig',
    'ArmMsgEndVelAccParamConfig',
    'ArmMsgCrashProtectionRatingConfig'
]




#!/usr/bin/env python3
# -*-coding:utf8-*-
from typing_extensions import (
    Literal,
)
class ArmMsgGripperTeachingPendantParamFeedback:
    '''
    msg_v2_feedback
    
    夹爪/示教器参数反馈指令(基于V1.5-2版本后)
    
    CAN ID:
        0x47E
    
    Args:
        teaching_range_per: 示教器行程系数反馈,[100~200]
        max_range_config: 夹爪/示教器最大控制行程限制值反馈,[0,70,100]
    
    位描述:
    
        Byte 0 示教器行程系数反馈, uint8 
            示教器行程系数反馈---100~200,单位(%)(默认100%)
            仅适用于设置主从臂的主臂，用于放大控制行程给从臂
        Byte 1 夹爪/示教器最大控制行程限制值反馈, uint8, 单位(mm)
            无效值---0
            小夹爪为---70mm
            大夹爪为---100mm
        Byte 2 保留
        Byte 3 保留
        Byte 4 保留
        Byte 5 保留
        Byte 6 保留
        Byte 7 保留
    '''
    '''
    msg_v2_feedback
    
    Gripper/Teaching Pendant Parameter Feedback(Based on version V1.5-2 and later)

    CAN ID:
        0x47E

    Args:
        teaching_range_per: Teaching pendant stroke coefficient setting.[100~200]
        max_range_config: Maximum control stroke limit setting for the gripper/teaching pendant.[0,70,100]

    Bit Description:
    
        Byte	Field	Type	Details
        Byte 0	Teaching range coefficient	uint8	Stroke coefficient setting: 100% to 200% (default: 100%).
                    Only applicable to the master arm in a master-slave setup to scale control range for the slave arm.
        Byte 1	Max control stroke limit	uint8	Invalid value: 0
                    Small gripper: 70 mm
                    Large gripper: 100 mm
        Byte 2	Reserved	-	Reserved for future use.
        Byte 3	Reserved	-	Reserved for future use.
        Byte 4	Reserved	-	Reserved for future use.
        Byte 5	Reserved	-	Reserved for future use.
        Byte 6	Reserved	-	Reserved for future use.
        Byte 7	Reserved	-	Reserved for future use.
    '''
    def __init__(self, 
                 teaching_range_per: int = 0, 
                 max_range_config: int = 0):
        self.teaching_range_per = teaching_range_per
        self.max_range_config = max_range_config

    def __str__(self):
        return (f"ArmMsgGripperTeachingPendantParamFeedback(\n"
                f"  teaching_range_per: {self.teaching_range_per},\n"
                f"  max_range_config: {self.max_range_config},\n"
                f")")

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-
from typing_extensions import (
    Literal,
)
class ArmMsgJointConfig:
    '''
    msg_v1_transmit
    
    关节设置指令

    CAN ID:
        0x475

    Args:
        joint_motor_num: 关节电机序号,[1, 7]
        set_motor_current_pos_as_zero: 设置当前位置为零点, 有效值-0xAE
        acc_param_config_is_effective_or_not: 加速度参数设置是否生效, 有效值-0xAE
        max_joint_acc: 最大关节加速度,单位0.01rad/s^2,输入范围\[0, 500\]-->[0 rad/s^2, 5.0 rad/s^2]
        clear_joint_err: 清除关节错误代码, 有效值-0xAE
    
    位描述:
    
        Byte 0: 关节电机序号 uint8, 值域 1-7
                1-6 代表关节驱动器序号；
                7 代表全部关节电机
        Byte 1: 设置N号电机当前位置为零点: uint8, 有效值-0xAE
        Byte 2: 加速度参数设置是否生效: uint8, 有效值-0xAE
        Byte 3: 最大关节加速度 H: uint16, 单位 0.01rad/s^2
        Byte 4: 最大关节加速度 L
        Byte 5: 清除关节错误代码: uint8, 有效值-0xAE
        Byte 6: 保留
        Byte 7: 保留
    '''
    '''
    msg_v1_transmit
    
    Joint Configuration Command

    CAN ID:
    0x475
    
    Args:
        joint_motor_num: Joint motor number.
            Value range: 1-6 represents individual joint motor numbers.
            Value 7 applies to all joint motors.
        set_motor_current_pos_as_zero: Command to set the current position of the specified joint motor as zero, with a valid value of 0xAE.
        acc_param_config_is_effective_or_not: Indicates whether the acceleration parameter configuration is effective, with a valid value of 0xAE.
        max_joint_acc: Maximum joint acceleration, unit: 0.01rad/s^2.Range is \[0, 500\]-->[0 rad/s^2, 5.0 rad/s^2]
        clear_joint_err: Command to clear joint error codes, with a valid value of 0xAE.

    Bit Description:

        Byte 0: Joint motor number (uint8).
                - 1-6: Corresponds to individual joint motor numbers.
                - 7: Represents all joint motors.
        Byte 1: Set the current position of the specified joint motor as zero (uint8).
                - Valid value: 0xAE.
        Byte 2: Determines if the acceleration parameter configuration is effective (uint8).
                - Valid value: 0xAE.
        Byte 3-4: Maximum joint acceleration (uint16).
                - Unit: 0.01rad/s^2.
                - Byte 3: High byte, Byte 4: Low byte.
        Byte 5: Clear joint error code (uint8).
                - Valid value: 0xAE.
        Byte 6: Reserved
        Byte 7: Reserved
    '''
    def __init__(self, 
                 joint_motor_num: Literal[1, 2, 3, 4, 5, 6, 7] = 7, 
                 set_motor_current_pos_as_zero: Literal[0x00, 0xAE] = 0, 
                 acc_param_config_is_effective_or_not: Literal[0x00, 0xAE] = 0,
                 max_joint_acc: int = 500,
                 clear_joint_err: Literal[0x00, 0xAE] = 0):
        if joint_motor_num not in [1, 2, 3, 4, 5, 6, 7]:
            raise ValueError(f"'joint_motor_num Value' {joint_motor_num} out of range [1, 2, 3, 4, 5, 6, 7]")
        if set_motor_current_pos_as_zero not in [0x00, 0xAE]:
            raise ValueError(f"'set_motor_current_pos_as_zero' Value {set_motor_current_pos_as_zero} out of range [0x00, 0xAE]")
        if acc_param_config_is_effective_or_not not in [0x00, 0xAE]:
            raise ValueError(f"'acc_param_config_is_effective_or_not' Value {acc_param_config_is_effective_or_not} out of range [0x00, 0xAE]")
        if not (0 <= max_joint_acc <= 500):
            raise ValueError(f"'max_joint_acc' Value {max_joint_acc} out of range 0-500.")
        if clear_joint_err not in [0x00, 0xAE]:
            raise ValueError(f"clear_joint_err' Value {clear_joint_err} out of range [0x00, 0xAE]")
        self.joint_motor_num = joint_motor_num
        self.set_motor_current_pos_as_zero = set_motor_current_pos_as_zero
        self.acc_param_config_is_effective_or_not = acc_param_config_is_effective_or_not
        self.max_joint_acc = max_joint_acc
        self.clear_joint_err = clear_joint_err

    def __str__(self):
        return (f"ArmMsgJointConfig(\n"
                f"  joint_motor_num: {self.joint_motor_num},\n"
                f"  set_motor_current_pos_as_zero: {self.set_motor_current_pos_as_zero},\n"
                f"  acc_param_config_is_effective_or_not: {self.acc_param_config_is_effective_or_not},\n"
                f"  max_joint_acc: {self.max_joint_acc}, {self.max_joint_acc*0.01:.2f}\n"
                f"  clear_joint_err: {self.clear_joint_err}\n"
                f")")

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-
from typing_extensions import (
    Literal,
)
class ArmMsgParamEnquiryAndConfig:
    '''
    msg_v1_transmit
    
    机械臂参数查询与设置指令
    
    CAN ID:
        0x477
    
    Args:
        param_enquiry: 参数查询
        param_setting: 参数设置
        data_feedback_0x48x: 0x48X报文反馈设置
        end_load_param_setting_effective: 末端负载参数设置是否生效
        set_end_load: 设置末端负载
    
    位描述:

        Byte 0: uint8,参数查询,查询末端 V/acc
                0x01,查询末端 V/acc
                0x02,查询碰撞防护等级
                0x03,查询当前轨迹索引
        Byte 1: uint8,参数设置,
                设置末端 V/acc 参数为初始值: 0x01
                设置全部关节限位、关节最大速度、关节加速度为默认值: 0x02
        Byte 2: uint8,0x48X 报文反馈设置,
                无效:0x00
                关闭周期反馈: 0x01;
                开启周期反馈: 0x02;
                开启后周期上报 1~6 号关节当前末端速度/加速度
        Byte 3: uint8,末端负载参数设置是否生效,有效值 : 0xAE
        Byte 4: uint8,设置末端负载,
                0x00 : 空载;
                0x01 : 半载;
                0x02 : 满载;
                0x03 : 无效
    '''
    '''
    msg_v1_transmit
    
    Robot Arm Parameter Query and Setting Command

    CAN ID:
        0x477

    Args:
        param_enquiry: Parameter query.
        param_setting: Parameter setting.
        data_feedback_0x48x: 0x48X message feedback setting.
        end_load_param_setting_effective: Whether the end-load parameter setting is effective.
        set_end_load: Set the end load.

    Bit Description:

        Byte 0: uint8, parameter query, used to query end velocity/acceleration or collision protection level:
            0x01: Query end velocity/acceleration.
            0x02: Query collision protection level.
            0x03: Query the current trajectory index.
        Byte 1: uint8, parameter setting, 
            set end velocity/acceleration parameters to initial values(0x01)
            Sets all joint limits, joint maximum speed, and joint acceleration to default values (0x02).
        Byte 2: uint8, 0x48X message feedback setting:
            0x00: Invalid.
            0x01: Disable periodic feedback.
            0x02: Enable periodic feedback (reports current end velocity/acceleration of joints 1~6).
        Byte 3: uint8, whether the end-load parameter setting is effective:
            Valid value: 0xAE.
        Byte 4: uint8, set the end load:
            0x00: No load.
            0x01: Half load.
            0x02: Full load.
            0x03: Invalid.
    '''
    def __init__(self, 
                 param_enquiry:Literal[0x00, 0x01, 0x02, 0x03] = 0x00, 
                 param_setting: Literal[0x00, 0x01, 0x02] = 0, 
                 data_feedback_0x48x: Literal[0x00, 0x01, 0x02] = 0x00,
                 end_load_param_setting_effective: Literal[0x00, 0xAE] = 0,
                 set_end_load: Literal[0x00, 0x01, 0x02, 0x03] = 0x03
                 ):
        if param_enquiry not in [0x00, 0x01, 0x02, 0x03]:
            raise ValueError(f"'param_enquiry' Value {param_enquiry} out of range [0x00, 0x01, 0x02, 0x03]")
        if param_setting not in [0x00, 0x01, 0x02]:
            raise ValueError(f"'param_setting' Value {param_setting} out of range [0x00, 0x01, 0x02]")
        if data_feedback_0x48x not in [0x00, 0x01, 0x02]:
            raise ValueError(f"'data_feedback_0x48x' Value {data_feedback_0x48x} out of range [0x00, 0x01, 0x02]")
        if end_load_param_setting_effective not in [0x00, 0xAE]:
            raise ValueError(f"'end_load_param_setting_effective' Value {end_load_param_setting_effective} out of range [0x00, 0xAE]")
        if set_end_load not in [0x00, 0x01, 0x02, 0x03]:
            raise ValueError(f"'set_end_load' Value {set_end_load} out of range [0x00, 0x01, 0x02, 0x03]")
        self.param_enquiry = param_enquiry
        self.param_setting = param_setting
        self.data_feedback_0x48x = data_feedback_0x48x
        self.end_load_param_setting_effective = end_load_param_setting_effective
        self.set_end_load = set_end_load

    def __str__(self):
        return (f"ArmMsgParamEnquiryAndConfig(\n"
                f"  param_enquiry: {self.param_enquiry},\n"
                f"  param_setting: {self.param_setting},\n"
                f"  data_feedback_0x48x: {self.data_feedback_0x48x},\n"
                f"  end_load_param_setting_effective: {self.end_load_param_setting_effective},\n"
                f"  set_end_load: {self.set_end_load}\n"
                f")")

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-

class ArmMsgStatus:
    '''
    msg_v2_feedback
    
    机械臂状态

    CAN ID:
        0x2A1

    Args:
        ctrl_mode: 控制模式
        arm_status: 机械臂状态
        mode_feed: 模式反馈
        teach_status: 示教状态
        motion_status: 运动状态
        trajectory_num: 当前运行轨迹点序号
        err_code: 故障码
    
    位描述:

        Byte 0:控制模式,uint8 
            0x00 待机模式
            0x01 CAN指令控制模式
            0x02 示教模式
            0x03 以太网控制模式
            0x04 wifi控制模式
            0x05 遥控器控制模式
            0x06 联动示教输入模式
            0x07 离线轨迹模式
        Byte 1:机械臂状态,uint8 
            0x00 正常
            0x01 急停
            0x02 无解
            0x03 奇异点
            0x04 目标角度超过限
            0x05 关节通信异常
            0x06 关节抱闸未打开
            0x07 机械臂发生碰撞
            0x08 拖动示教时超速
            0x09 关节状态异常
            0x0A 其它异常
            0x0B 示教记录
            0x0C 示教执行
            0x0D 示教暂停
            0x0E 主控NTC过温
            0x0F 释放电阻NTC过温
        Byte 2:模式反馈,uint8 
            0x00 MOVE P
            0x01 MOVE J
            0x02 MOVE L
            0x03 MOVE C
            0x04 MOVE M ---基于V1.5-2版本后
        Byte 3:示教状态,uint8 
            0x00 关闭
            0x01 开始示教记录（进入拖动示教模式）
            0x02 结束示教记录（退出拖动示教模式）
            0x03 执行示教轨迹（拖动示教轨迹复现）
            0x04 暂停执行
            0x05 继续执行（轨迹复现继续）
            0x06 终止执行
            0x07 运动到轨迹起点
        Byte 4:运动状态,uint8 
            0x00 到达指定点位
            0x01 未到达指定点位
        Byte 5:当前运行轨迹点序号,uint8_t
            0~255 (离线轨迹模式下反馈)
        Byte 6:故障码*,uint16
            bit[0]      1号关节通信异常(0:正常 1:异常)
            bit[1]      2号关节通信异常(0:正常 1:异常)
            bit[2]      3号关节通信异常(0:正常 1:异常)
            bit[3]      4号关节通信异常(0:正常 1:异常)
            bit[4]      5号关节通信异常(0:正常 1:异常)
            bit[5]      6号关节通信异常(0:正常 1:异常)
            bit[6]      保留(Reserved)
            bit[7]      保留(Reserved)
        Byte 7:故障码*,uint16
            bit[0]      1号关节角度超限位(0:正常 1:异常)
            bit[1]      2号关节角度超限位(0:正常 1:异常)
            bit[2]      3号关节角度超限位(0:正常 1:异常)
            bit[3]      4号关节角度超限位(0:正常 1:异常)
            bit[4]      5号关节角度超限位(0:正常 1:异常)
            bit[5]      6号关节角度超限位(0:正常 1:异常)
            bit[6]      保留
            bit[7]      保留
    '''
    '''
    msg_v2_feedback
    
    Robot Arm Status

    CAN ID: 
        0x2A1

    Arguments:
        ctrl_mode: Control mode
        arm_status: Robot arm status
        mode_feed: Mode feedback
        teach_status: Teaching status
        motion_status: Motion status
        trajectory_num: Current trajectory point number
        err_code: Error code
    
    Bit Description:

        Byte 0: Control mode, uint8
            0x00: Standby mode
            0x01: CAN instruction control mode
            0x02: Teaching mode
            0x03: Ethernet control mode
            0x04: Wi-Fi control mode
            0x05: Remote control mode
            0x06: Linkage teaching input mode
            0x07: Offline trajectory mode
        Byte 1: Robot arm status, uint8
            0x00: Normal
            0x01: Emergency stop
            0x02: No solution
            0x03: Singularity point
            0x04: Target angle exceeds limit
            0x05: Joint communication exception
            0x06: Joint brake not released
            0x07: Collision occurred
            0x08: Overspeed during teaching drag
            0x09: Joint status abnormal
            0x0A: Other exception
            0x0B: Teaching record
            0x0C: Teaching execution
            0x0D: Teaching pause
            0x0E: Main controller NTC over temperature
            0x0F: Release resistor NTC over temperature
        Byte 2: Mode feedback, uint8
            0x00: MOVE P
            0x01: MOVE J
            0x02: MOVE L
            0x03: MOVE C
            0x04: MOVE M
        Byte 3: Teaching status, uint8
            0x00: Off
            0x01: Start teaching record (enter drag teaching mode)
            0x02: End teaching record (exit drag teaching mode)
            0x03: Execute teaching trajectory (reproduce drag teaching trajectory)
            0x04: Pause execution
            0x05: Continue execution (continue trajectory reproduction)
            0x06: Terminate execution
            0x07: Move to trajectory starting point
        Byte 4: Motion status, uint8
            0x00: Reached the target position
            0x01: Not yet reached the target position
        Byte 5: Current trajectory point number, uint8_t
            0~255 (feedback in offline trajectory mode)
        Byte 6: Error code, uint16
            bit[0]: Joint 1 communication exception (0: normal, 1: abnormal)
            bit[1]: Joint 2 communication exception (0: normal, 1: abnormal)
            bit[2]: Joint 3 communication exception (0: normal, 1: abnormal)
            bit[3]: Joint 4 communication exception (0: normal, 1: abnormal)
            bit[4]: Joint 5 communication exception (0: normal, 1: abnormal)
            bit[5]: Joint 6 communication exception (0: normal, 1: abnormal)
            bit[6]: Reserved
            bit[7]: Reserved
        Byte 7: Error code, uint16
            bit[0]: Joint 1 angle limit exceeded (0: normal, 1: abnormal)
            bit[1]: Joint 2 angle limit exceeded (0: normal, 1: abnormal)
            bit[2]: Joint 3 angle limit exceeded (0: normal, 1: abnormal)
            bit[3]: Joint 4 angle limit exceeded (0: normal, 1: abnormal)
            bit[4]: Joint 5 angle limit exceeded (0: normal, 1: abnormal)
            bit[5]: Joint 6 angle limit exceeded (0: normal, 1: abnormal)
            bit[6]: Reserved
            bit[7]: Reserved
    '''
    
    def __init__(self,
                 ctrl_mode: int = 0,
                 arm_status: int = 0,
                 mode_feed: int = 0,
                 teach_status: int = 0,
                 motion_status: int = 0,
                 trajectory_num: int = 0,
                 err_code: int = 0):
        self.ctrl_mode: int = ctrl_mode       #控制模式
        self.arm_status: int = arm_status      #机械臂状态
        self.mode_feed: int = mode_feed       #模式反馈
        self.teach_status: int = teach_status    #示教状态
        self.motion_status: int = motion_status   #运动状态
        self.trajectory_num: int = trajectory_num  #当前运行轨迹点序号
        self._err_code = err_code         #故障码
        self.err_status = self.ErrStatus()#故障码

    class ErrStatus:
        def __init__(self):
            self.joint_1_angle_limit = False
            self.joint_2_angle_limit = False
            self.joint_3_angle_limit = False
            self.joint_4_angle_limit = False
            self.joint_5_angle_limit = False
            self.joint_6_angle_limit = False
            self.communication_status_joint_1 = False
            self.communication_status_joint_2 = False
            self.communication_status_joint_3 = False
            self.communication_status_joint_4 = False
            self.communication_status_joint_5 = False
            self.communication_status_joint_6 = False
            
        def __str__(self):
            return (f" Joint 1 Angle Limit Status: {self.joint_1_angle_limit}\n"
                    f" Joint 2 Angle Limit Status: {self.joint_2_angle_limit}\n"
                    f" Joint 3 Angle Limit Status: {self.joint_3_angle_limit}\n"
                    f" Joint 4 Angle Limit Status: {self.joint_4_angle_limit}\n"
                    f" Joint 5 Angle Limit Status: {self.joint_5_angle_limit}\n"
                    f" Joint 6 Angle Limit Status: {self.joint_6_angle_limit}\n"
                    f" Joint 1 Communication Status: {self.communication_status_joint_1}\n"
                    f" Joint 2 Communication Status: {self.communication_status_joint_2}\n"
                    f" Joint 3 Communication Status: {self.communication_status_joint_3}\n"
                    f" Joint 4 Communication Status: {self.communication_status_joint_4}\n"
                    f" Joint 5 Communication Status: {self.communication_status_joint_5}\n"
                    f" Joint 6 Communication Status: {self.communication_status_joint_6}\n")

    @property
    def err_code(self):
        return self._err_code

    @err_code.setter
    def err_code(self, value: int):
        if not (0 <= value < 2**16):
            raise ValueError("err_code must be an 8-bit integer between 0 and 255.")
        self._err_code = value
        # Update err_status based on the err_code bits
        self.err_status.communication_status_joint_1 = bool(value & (1 << 0))
        self.err_status.communication_status_joint_2 = bool(value & (1 << 1))
        self.err_status.communication_status_joint_3 = bool(value & (1 << 2))
        self.err_status.communication_status_joint_4 = bool(value & (1 << 3))
        self.err_status.communication_status_joint_5 = bool(value & (1 << 4))
        self.err_status.communication_status_joint_6 = bool(value & (1 << 5))
        self.err_status.communication_status_joint_1 = bool(value & (1 << 8))
        self.err_status.communication_status_joint_2 = bool(value & (1 << 9))
        self.err_status.communication_status_joint_3 = bool(value & (1 << 10))
        self.err_status.communication_status_joint_4 = bool(value & (1 << 11))
        self.err_status.communication_status_joint_5 = bool(value & (1 << 12))
        self.err_status.communication_status_joint_6 = bool(value & (1 << 13))

    def __str__(self):
        return (f"Control Mode: {self.ctrl_mode}\n"
                f"Arm Status: {self.arm_status}\n"
                f"Mode Feed: {self.mode_feed}\n"
                f"Teach Status: {self.teach_status}\n"
                f"Motion Status: {self.motion_status}\n"
                f"Trajectory Num: {self.trajectory_num}\n"
                f"Error Code: {self._err_code}\n"
                f"Error Status: \n{self.err_status}\n")

    def __repr__(self):
        return self.__str__()


#!/usr/bin/env python3
# -*-coding:utf8-*-
from typing_extensions import (
    Literal,
)
class ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd:
    '''
    msg_v1_feedback
    
    反馈当前电机限制角度/最大速度
    
    CAN ID:
        0x473
    
    Args:
        motor_num: 关节电机序号
        max_angle_limit: 最大角度限制
        min_angle_limit: 最小角度限制
        max_joint_spd: 最大关节速度

    位描述:

        Byte 0: 关节电机序号, uint8
        Byte 1: 最大角度限制H,uint16, 单位 0.1度
        Byte 2: 最大角度限制L
        Byte 3: 最小角度限制H, uint16, 单位 0.1度
        Byte 4: 最小角度限制L
        Byte 5: 最大关节速度H, uint16, 单位 0.001rad/s
        Byte 6: 最大关节速度L
        Byte 7: 保留
    '''
    '''
    msg_v1_feedback
    
    Feedback on Current Motor Angle Limits/Maximum Speed

    CAN ID:
        0x473

    Args:
        motor_num: Joint motor number.
        max_angle_limit: Maximum angle limit.
        min_angle_limit: Minimum angle limit.
        max_joint_spd: Maximum joint speed.
    
    Bit Description:

        Byte 0: Joint Motor Index, uint8
        Byte 1: Maximum Angle Limit (High Byte), uint16, unit: 0.1°
        Byte 2: Maximum Angle Limit (Low Byte)
        Byte 3: Minimum Angle Limit (High Byte), uint16, unit: 0.1°
        Byte 4: Minimum Angle Limit (Low Byte)
        Byte 5: Maximum Joint Speed (High Byte), uint16, unit: 0.001 rad/s
        Byte 6: Maximum Joint Speed (Low Byte)
        Byte 7: Reserved
    '''
    def __init__(self, 
                 motor_num: Literal[0, 1, 2, 3, 4, 5, 6] = 0, 
                 max_angle_limit: int = 0, 
                 min_angle_limit: int = 0,
                 max_joint_spd: int = 0):
        if motor_num not in [0, 1, 2, 3, 4, 5, 6]:
            raise ValueError(f"'motor_num' Value {motor_num} out of range [1, 2, 3, 4, 5, 6]")
        self.motor_num = motor_num
        self.max_angle_limit = max_angle_limit
        self.min_angle_limit = min_angle_limit
        self.max_joint_spd = max_joint_spd

    def __str__(self):
        return (f"ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd(\n"
                f"  motor_num: {self.motor_num},\n"
                f"  max_angle_limit: {self.max_angle_limit}, {self.max_angle_limit * 0.1:.1f},\n"
                f"  min_angle_limit: {self.min_angle_limit}, {self.min_angle_limit * 0.1:.1f},\n"
                f"  max_joint_spd: {self.max_joint_spd}, {self.max_joint_spd * 0.001:.3f}\n"
                f")")

    def __repr__(self):
        return self.__str__()

class ArmMsgFeedbackAllCurrentMotorAngleLimitMaxSpd:
    '''
    反馈所有电机限制角度/最大速度
    
    CAN ID:
        0x473
    
    :param m1: 电机1的限制角度/最大速度
    :param m2: 电机2的限制角度/最大速度
    :param m3: 电机3的限制角度/最大速度
    :param m4: 电机4的限制角度/最大速度
    :param m5: 电机5的限制角度/最大速度
    :param m6: 电机6的限制角度/最大速度
    '''
    '''
    Feedback on Current Motor Angle Limits/Maximum Speed

    CAN ID:
        0x473

    :param m1: Limit angle/maximum speed for motor 1.
    :param m2: Limit angle/maximum speed for motor 2.
    :param m3: Limit angle/maximum speed for motor 3.
    :param m4: Limit angle/maximum speed for motor 4.
    :param m5: Limit angle/maximum speed for motor 5.
    :param m6: Limit angle/maximum speed for motor 6.
    '''
    def __init__(self, 
                 m1:ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd = ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd(0,0,0,0),
                 m2:ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd = ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd(0,0,0,0),
                 m3:ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd = ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd(0,0,0,0),
                 m4:ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd = ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd(0,0,0,0),
                 m5:ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd = ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd(0,0,0,0),
                 m6:ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd = ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd(0,0,0,0)):
        self.__m = [ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd(0,0,0,0), m1, m2, m3, m4, m5, m6]
        self.motor = [ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd() for _ in range(7)]
        self.motor[0] = ArmMsgFeedbackCurrentMotorAngleLimitMaxSpd(0,0,0,0)
    
    def assign(self):
        for i in range(1,7):
            if(self.__m[i].motor_num != 0):
                self.motor[i] = self.__m[i]

    def __str__(self):
        return (f"{self.motor[1]}\n"
                f"{self.motor[2]}\n"
                f"{self.motor[3]}\n"
                f"{self.motor[4]}\n"
                f"{self.motor[5]}\n"
                f"{self.motor[6]}\n"
                f")")

    def __repr__(self):
        return self.__str__()


#!/usr/bin/env python3
# -*-coding:utf8-*-
from typing_extensions import (
    Literal,
)

class ArmMsgInstructionResponseConfig:
    '''
    msg_v1_transmit
    
    设置指令应答

    CAN ID:
        0x476

    Args:
        instruction_index: 应答指令索引
        zero_config_success_flag: 零点是否设置成功
    
    位描述:
    
        Byte 0: uint8, 应答指令索引
                取设置指令 id 最后一个字节
                例如：应答 0x471 设置指令时此位填充0x71
        Byte 1: uint8, 零点是否设置成功
                零点成功设置 : 0x01
                设置失败/未设置: 0x00
                仅在关节设置指令--成功设置 N 号电机当前位置为零点时应答 0x01
    '''
    '''
    msg_v1_transmit
    
    Set Command Response

    CAN ID:
        0x476

    Args:
        instruction_index: Response instruction index.
        zero_config_success_flag: Whether the zero-point configuration was successful.

    Bit Description:

        Byte 0: uint8, response instruction index.
            Fill in the last byte of the set command ID.
                Example: Responding to the 0x471 set command, this byte will be 0x71.
        Byte 1: uint8, zero-point configuration success flag.
            0x01: Zero-point successfully set.
            0x00: Failed to set/Not set.
    '''
    def __init__(self,
                 instruction_index: int = 0,
                 zero_config_success_flag: Literal[0x00, 0x01] =0 ):
        if zero_config_success_flag not in [0x00, 0x01]:
            raise ValueError(f"'zero_config_success_flag' Value {zero_config_success_flag} out of range [0x01, 0x02]")
        self.instruction_index = instruction_index
        self.zero_config_success_flag = zero_config_success_flag

    def __str__(self):
        return (f"ArmMsgInstructionResponseConfig(\n"
                f"  instruction_index: {self.instruction_index },\n"
                f"  zero_config_success_flag: {self.zero_config_success_flag },\n"
                f")")

    def __repr__(self):
        return self.__str__()




from .hardware_port.can_encapsulation import C_STD_CAN
from .base.piper_base import C_PiperBase
from .monitor.fps import C_FPSCounter
from .piper_msgs.msg_v1 import *
from .protocol.protocol_v1 import *
from .piper_msgs.msg_v2 import *
from .protocol.protocol_v2 import *
from .kinematics.piper_fk import C_PiperForwardKinematics
from .protocol.piper_protocol_base import C_PiperParserBase
from .interface.piper_interface import C_PiperInterface
from .interface.piper_interface_v1 import C_PiperInterface_V1
from .interface.piper_interface_v2 import C_PiperInterface_V2

__all__ = [
    'C_PiperParserBase',
    'C_FPSCounter',
    'C_PiperForwardKinematics',
    'C_STD_CAN',
    'C_PiperBase',
    'C_PiperInterface',
    'C_PiperInterface_V1',
    'C_PiperInterface_V2'
]



#!/usr/bin/env python3
# -*-coding:utf8-*-

class ArmMsgEndPoseFeedBack():
    '''
    msg_v2_feedback
    
    机械臂末端姿态反馈,单位0.001mm
    
    CAN ID: 
        0x152、0x153、0x154
    
    Args:
        X_axis: X坐标
        Y_axis: Y坐标
        Z_axis: Z坐标
        RX_axis: RX角度
        RY_axis: RY角度
        RZ_axis: RZ角度
    '''
    '''
    msg_v2_feedback
    
    End-Effector Pose Feedback for the Robotic Arm, unit: 0.001 mm.
    
    CAN ID: 
        0x152、0x153、0x154
    
    Args:
        X_axis: X-coordinate.
        Y_axis: Y-coordinate.
        Z_axis: Z-coordinate.
        RX_axis: Rotation angle around the X-axis (RX).
        RY_axis: Rotation angle around the Y-axis (RY).
        RZ_axis: Rotation angle around the Z-axis (RZ).
    '''
    def __init__(self, 
                 X_axis: int = 0, 
                 Y_axis: int = 0, 
                 Z_axis: int = 0, 
                 RX_axis: int = 0, 
                 RY_axis: int = 0, 
                 RZ_axis: int = 0):
        self.X_axis = X_axis
        self.Y_axis = Y_axis
        self.Z_axis = Z_axis
        self.RX_axis = RX_axis
        self.RY_axis = RY_axis
        self.RZ_axis = RZ_axis

    def __str__(self):
        # 将角度乘以0.001，并保留三位小数
        end_pose = [
            (" X_axis ", self.X_axis, self.X_axis * 0.001),
            (" Y_axis ", self.Y_axis, self.Y_axis * 0.001),
            (" Z_axis ", self.Z_axis, self.Z_axis * 0.001),
            (" RX_axis ", self.RX_axis, self.RX_axis * 0.001),
            (" RY_axis ", self.RY_axis, self.RY_axis * 0.001),
            (" RZ_axis ", self.RZ_axis, self.RZ_axis * 0.001)
        ]

        # 生成格式化字符串，保留三位小数
        formatted_ = "\n".join([f"{name}: {pose}, {pose_f:.3f}" for name, pose, pose_f in end_pose])
        
        return f"ArmMsgEndPoseFeedBack:\n{formatted_}"
    
    def __repr__(self):
        return self.__str__()


#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
from typing import (
    Optional,
)
import time
from piper_sdk import *

def enable_fun(piper:C_PiperInterface):
    '''
    使能机械臂并检测使能状态,尝试5s,如果使能超时则退出程序
    '''
    enable_flag = False
    # 设置超时时间（秒）
    timeout = 5
    # 记录进入循环前的时间
    start_time = time.time()
    elapsed_time_flag = False
    while not (enable_flag):
        elapsed_time = time.time() - start_time
        print("--------------------")
        enable_flag = piper.GetArmLowSpdInfoMsgs().motor_1.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_2.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_3.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_4.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_5.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_6.foc_status.driver_enable_status
        print("使能状态:",enable_flag)
        piper.EnableArm(7)
        piper.GripperCtrl(0,1000,0x01, 0)
        print("--------------------")
        # 检查是否超过超时时间
        if elapsed_time > timeout:
            print("超时....")
            elapsed_time_flag = True
            enable_flag = True
            break
        time.sleep(1)
        pass
    if(elapsed_time_flag):
        print("程序自动使能超时,退出程序")
        exit(0)

if __name__ == "__main__":
    piper = C_PiperInterface("can0")
    piper.ConnectPort()
    piper.EnableArm(7)
    enable_fun(piper=piper)
    # piper.DisableArm(7)
    piper.GripperCtrl(0,1000,0x01, 0)
    factor = 57324.840764 #1000*180/3.14
    # factor =1
    position = [0,0,0,0,0,0,0]
    count = 0
    while True:
        print(piper.GetArmStatus())
        import time
        count  = count + 1
        # print(count)
        if(count == 0):
            print("1-----------")
            position = [0,0,0,0,0,0,0]
            # position = [0.2,0.2,-0.2,0.3,-0.2,0.5,0.08]
        elif(count == 800):
            print("2-----------")
            position = [0.2,0.2,-0.2,0.3,-0.2,0.5,0.08]
            # position = [0,0,0,0,0,0,0]
            # position = [-8524,104705,-78485,-451,-5486,29843,0]
        elif(count == 1600):
            print("1-----------")
            position = [0,0,0,0,0,0,0]
            # position = [0.2,0.2,-0.2,0.3,-0.2,0.5,0.08]
            count = 0
        
        joint_0 = round(position[0]*factor)
        joint_1 = round(position[1]*factor)
        joint_2 = round(position[2]*factor)
        joint_3 = round(position[3]*factor)
        joint_4 = round(position[4]*factor)
        joint_5 = round(position[5]*factor)
        joint_6 = round(position[6]*1000*1000)
        # piper.MotionCtrl_1()
        piper.MotionCtrl_2(0x01, 0x01, 30, 0x00)
        piper.JointCtrl(joint_0, joint_1, joint_2, joint_3, joint_4, joint_5)
        piper.GripperCtrl(abs(joint_6), 1000, 0x01, 0)
        time.sleep(0.005)
        pass